package com.gct.edge.actuator.config;


import feign.RequestInterceptor;
import feign.RequestTemplate;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.EnvironmentAware;
import org.springframework.context.annotation.AnnotationConfigApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.env.Environment;
import org.springframework.web.client.RestTemplate;

import java.util.Arrays;
import java.util.List;
import java.util.concurrent.ThreadLocalRandom;


/**
 * feign 调用配置，统一添加token请求头
 * @author: NewMeanning
 * @create: 2021-01-29 00:10
 **/
@Slf4j
@Configuration
public class FeignConfig implements RequestInterceptor, EnvironmentAware {

    private static String token;

    private static Environment env;

    @Bean(name = "restTemplate")
    RestTemplate getRestTemplate(){
        return new RestTemplate();
    }

    @Override
    public void apply(RequestTemplate requestTemplate) {
        requestTemplate.header("token",token);//请求头添加token
    }

    @Override
    public void setEnvironment(Environment environment) {
        //获取配置的认证中心主机地址  todo 升级为自动从Nacos中读取，目前这里读取不到
        this.env = environment;
        List<String> hostList = Arrays.asList(this.env.getProperty("auth.hostList").split(","));
        //手动获取RestTemplate 实例
        AnnotationConfigApplicationContext applicationContext = new AnnotationConfigApplicationContext();
        applicationContext.register(RestTemplate.class);
        applicationContext.refresh();
        RestTemplate restTemplate= (RestTemplate) applicationContext.getBean("restTemplate");
        //自定义轮询方式负载
        int index = ThreadLocalRandom.current().nextInt(hostList.size());
        String url = "http://"+hostList.get(index)+"/token";
        token = restTemplate.getForObject(url,String.class);
        log.info("token:{}",token);
    }
}
