package com.gct.edge.actuator.config;

import com.gct.common.security.annotation.EnableGCTCacheSecurityCheck;
import com.gct.common.security.config.WebSecurityConfigProperties;
import com.gct.common.security.core.CheckTokenCaching;
import com.gct.common.security.core.CustomAccessDeniedHandler;
import com.gct.common.security.core.JwtAuthenticationFilter;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.config.annotation.method.configuration.EnableGlobalMethodSecurity;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.config.annotation.web.configurers.ExpressionUrlAuthorizationConfigurer;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.web.access.AccessDeniedHandler;
import org.springframework.security.web.authentication.www.BasicAuthenticationFilter;

@Configuration
@EnableWebSecurity
@EnableGlobalMethodSecurity(prePostEnabled = true)
@EnableGCTCacheSecurityCheck
@EnableConfigurationProperties(WebSecurityConfigProperties.class)
public class GCTServiceSecurityConfig extends WebSecurityConfigurerAdapter {
    private JwtAuthenticationFilter jwtAuthenticationFilter;
    private AccessDeniedHandler accessDeniedHandler;
    public AuthenticationManager authenticationManager;
    private CheckTokenCaching checkTokenCaching;

    @Autowired
//    @Qualifier("spring.security.gct-com.gct.common.security.config.WebSecurityConfigProperties")
    WebSecurityConfigProperties webSecurityConfigProperties;

    @Autowired(required = false)
    public GCTServiceSecurityConfig(CheckTokenCaching checkTokenCaching) throws Exception {
        this.accessDeniedHandler = new CustomAccessDeniedHandler();
        this.checkTokenCaching = checkTokenCaching;
    }

    @Override
    protected void configure(HttpSecurity http) throws Exception {
        authenticationManager = authenticationManager();
        jwtAuthenticationFilter = new JwtAuthenticationFilter(authenticationManager, checkTokenCaching);
        ExpressionUrlAuthorizationConfigurer<HttpSecurity>.ExpressionInterceptUrlRegistry registry = http
                .csrf().disable()
                .authorizeRequests();
        String[] ams = {"/*"};
        if (webSecurityConfigProperties.getAntMatchers() != null && webSecurityConfigProperties.getAntMatchers().length > 0)
            ams = webSecurityConfigProperties.getAntMatchers();

        for (String am : ams) {
            registry = registry.antMatchers(am).permitAll();
            LoggerFactory.getLogger(this.getClass()).warn("\033[1;34mregistry ant matcher to permitAll:{}\033[0m ", am);
        }
        /*.antMatchers("/webjars/**").permitAll()
                .antMatchers("/swagger-ui.html").permitAll()
                .antMatchers("/swagger-resources/**").permitAll()
                .antMatchers("/v2/api-docs").permitAll()
                // 服务监控
                .antMatchers("/actuator/**").permitAll()
        */        // 其他所有请求需要身份认证
        registry.anyRequest().authenticated()
                .and().exceptionHandling().accessDeniedHandler(accessDeniedHandler)
                .and()
                //禁用session
                .sessionManagement().sessionCreationPolicy(SessionCreationPolicy.STATELESS)
                .and()
                //添加自定义过滤器
                .addFilterAt(jwtAuthenticationFilter, BasicAuthenticationFilter.class);
    }
/*    @Autowired
    public void configureGlobal(AuthenticationManagerBuilder auth) throws Exception {
        BCryptPasswordEncoder bCryptPasswordEncoder = new BCryptPasswordEncoder();
        auth.inMemoryAuthentication() //认证信息存储到内存中
                .passwordEncoder(bCryptPasswordEncoder);
                //.withUser("user").password(bCryptPasswordEncoder.encode("123456")).roles("ADMIN");
    }*/

    @Bean
    public AuthenticationManager authenticationManager() throws Exception {
        return super.authenticationManager();
    }

}
