package com.gct.edge.actuator.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.gct.common.core.result.Result;
import com.gct.common.util.StringUtil;
import com.gct.edge.actuator.service.DiagnoseService;
import com.gct.edge.base.entity.TableNameEnum;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

/**
 * dec: Production
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/10/24 下午1:53
 */
@RestController
@RequestMapping("/module/diagnose")
public class DiagnoseController {

    @Autowired
    DiagnoseService diagnoseService;

    @PreAuthorize("hasAuthority('module:view:read')")
    @GetMapping("/getDiagnoseResultByDate")
    public Result getDiagnoseResultByDate(@RequestParam String moduleKey,
                                          @RequestParam String date,
                                          @RequestParam String wellId,
                                          @RequestParam Integer curPage,
                                          @RequestParam Integer pageSize
    ) {
        return diagnoseService.getByDate(moduleKey, wellId, date, curPage, pageSize);
    }

    @PreAuthorize("hasAuthority('module:view:write')")
    @GetMapping("/deleteResultByTimeRange")
    public Result deleteResultByTimeRange(@RequestParam String moduleKey,
                                          @RequestParam String beginTime,
                                          @RequestParam String endTime
    ) {
        return diagnoseService.deleteResultByTimeRange(moduleKey, beginTime, endTime);
    }

    @PreAuthorize("hasAuthority('module:view:write')")
    @PostMapping("/uploadDiagnoseResult")
    public Result uploadDiagnoseResult(@RequestParam MultipartFile file, @RequestParam String moduleKey) {
        try {
            return diagnoseService.loadDataFromFile(file.getInputStream(), moduleKey, TableNameEnum.DIAGNOSE_RESULT_REAL_TIME);
        } catch (Exception e) {
            return Result.error(file.getOriginalFilename() + " upload happened a error cause by: " + e.getMessage());
        }
    }

    @PreAuthorize("hasAuthority('module:view:read')")
    @GetMapping("/getGtOverlayData")
    public Result getGtOverlayData(@RequestParam String moduleKey, @RequestParam String wellId,
                                   @RequestParam String beginDate, @RequestParam String endDate,
                                   @RequestParam Integer splitCount) {
        if (StringUtil.isBlankOrEmpty(moduleKey)) return Result.error("参数条件不足");
        JSONArray ja = diagnoseService.getGtOverlayData(moduleKey, wellId, beginDate, endDate, splitCount);
        return ja.size() > 30 ? Result.error("叠加后产生的结果大于30，可能导致浏览器卡死，请重新输入分割参数") : Result.ok(ja);
    }

    @PreAuthorize("hasAuthority('module:view:read')")
    @GetMapping("/getGtOverlayDataLazy")
    @ApiOperation(value = "getGtOverlayDataLazy(根据参数条件查询叠加功图--分页查询,懒加载方式)", notes = "功图叠加")
    public Result getGtOverlayDataLazy(@RequestParam String moduleKey, @RequestParam String wellId,
                                       @RequestParam String beginDate, @RequestParam String endDate,
                                       @ApiParam("叠加数,也就相当于pageSize") @RequestParam Integer splitCount,
                                       @ApiParam("当前第几个叠加图,也就相当于curPage") @RequestParam Integer curIndex) {
        if (StringUtil.isBlankOrEmpty(moduleKey)) return Result.error("参数条件不足");
        if (splitCount != null && splitCount >= 20000) return Result.error("超过叠加上限");
        JSONObject ja = diagnoseService.getGtOverlayDataLazy(moduleKey, wellId, beginDate, endDate, splitCount,curIndex);
        return Result.ok(ja);
    }

}
