package com.gct.edge.actuator.controller;

import com.gct.common.core.result.Result;
import com.gct.common.util.DateTimeUtil;
import com.gct.edge.actuator.config.SyncModuleDataProperties;
import com.gct.edge.actuator.service.AoidProdStateService;
import com.gct.edge.actuator.service.DiagnoseService;
import com.gct.edge.actuator.service.SyncModuleConfigService;
import com.gct.edge.actuator.service.YieldService;
import com.gct.edge.actuator.util.SshUtil;
import com.gct.edge.base.entity.TableNameEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

import static com.gct.edge.base.entity.TableNameEnum.*;

/**
 * @author xusirui 2021/11/1
 */
@RestController
@RequestMapping("/module/fileUpload")
public class FileUploadController {

    @Autowired
    SyncModuleDataProperties properties;

    @Autowired
    YieldService yieldService;

    @Autowired
    DiagnoseService diagnoseService;

    @Autowired
    SyncModuleConfigService syncModuleConfigService;

    @Autowired
    AoidProdStateService aoidProdStateService;

    @Autowired
    SyncModuleDataProperties syncModuleDataProperties;

    //    @PreAuthorize("hasAuthority('module:view:read')")
    @GetMapping("/loadRemoteFile")
    public Result loadRemoteFile(@RequestParam String moduleKey, @RequestParam String ipAddress, @RequestParam Integer port,
                                 @RequestParam String authUser, @RequestParam String authPassword, @RequestParam String remotePath) {
        List<TableNameEnum> tableKeys = new ArrayList<>();
        String[] readTableFromModule = syncModuleDataProperties.getReadTableFromModule();
        for (int i = 0; i < readTableFromModule.length; i++) {
            if (Objects.isNull(tableNameOf(readTableFromModule[i]))) continue;

            tableKeys.add(tableNameOf(readTableFromModule[i]));
        }
        for (int i = 0; i < tableKeys.size(); i++) {
            TableNameEnum name = tableKeys.get(i);
            try {
                File remoteFile = SshUtil.getRemoteFile(authUser, authPassword, ipAddress, port, remotePath + "/" + name.getActualFileName(), properties.getPath() + moduleKey + "/" + name.getActualFileName() + "-" + new Date().getTime());
                switch (name) {
                    case DIAGNOSE_RESULT_REAL_TIME:
                        diagnoseService.loadDataFromFile(new FileInputStream(remoteFile), moduleKey, DIAGNOSE_RESULT_REAL_TIME);
                        break;
                    case YIELD_REAL_TIME:
                        yieldService.loadDataFromFile(new FileInputStream(remoteFile), moduleKey, YIELD_REAL_TIME);
                        break;
                    case AOID_PROD_STATE:
                        aoidProdStateService.loadDataFromFile(new FileInputStream(remoteFile), moduleKey, AOID_PROD_STATE);
                        break;
                }
                if (remoteFile.exists()) remoteFile.delete();
            } catch (Exception e) {
                return Result.error("load data from remote file: " + name.getActualFileName() + " failed, cause by " + e.getMessage());
            }
        }
        return Result.ok("导入成功");
    }

    @PreAuthorize("hasAuthority('module:view:write')")
    @GetMapping("/loadLocalFile")
    public Result loadLocalFile(@RequestParam String path, @RequestParam String moduleKey, @RequestParam String uploadChoose) {
        try {
            if (DIAGNOSE_RESULT_REAL_TIME.getLabel().equals(uploadChoose)) {
                return diagnoseService.loadDataFromFile(new FileInputStream(path), moduleKey, DIAGNOSE_RESULT_REAL_TIME);
            } else if (YIELD_REAL_TIME.getLabel().equals(uploadChoose)) {
                return yieldService.loadDataFromFile(new FileInputStream(path), moduleKey, YIELD_REAL_TIME);
            } else {
                return Result.error("暂不支持该类型的文件");
            }
        } catch (Exception e) {
            return Result.error(path + " upload happened a error cause by: " + e.getMessage());
        }
    }

    @PreAuthorize("hasAuthority('module:view:read')")
    @GetMapping("/getUploadType")
    public Result getUploadType() {
        return Result.ok(Arrays.stream(values()).map(TableNameEnum::getLabel).collect(Collectors.toList()));
    }

    @PreAuthorize("hasAuthority('module:view:write')")
    @PostMapping("/upload")
    public Result upload(@RequestParam MultipartFile file, @RequestParam String moduleKey, @RequestParam String uploadChoose) {
        try {
            if (DIAGNOSE_RESULT_REAL_TIME.getLabel().equals(uploadChoose)) {
                return diagnoseService.loadDataFromFile(file.getInputStream(), moduleKey, DIAGNOSE_RESULT_REAL_TIME);
            } else if (YIELD_REAL_TIME.getLabel().equals(uploadChoose)) {
                return yieldService.loadDataFromFile(file.getInputStream(), moduleKey, YIELD_REAL_TIME);
            } else {
                return Result.error("暂不支持该类型的文件");
            }
        } catch (Exception e) {
            return Result.error(file.getOriginalFilename() + " upload happened a error cause by: " + e.getMessage());
        }
    }

    @GetMapping("/syncModuleConfig")
    public Result syncModuleConfig(@RequestParam String moduleKey, @RequestParam String proTime, @RequestParam String ipAddress,
                                   @RequestParam Integer port, @RequestParam String authUser, @RequestParam String authPassword,
                                   @RequestParam String remotePath, @RequestParam String tableName, @RequestParam String actualFileName) {
        return syncModuleConfigService.syncModuleConfig(moduleKey, DateTimeUtil.formatDateTime(proTime), ipAddress, port, authUser, authPassword, remotePath, tableName, actualFileName);


    }

}
