package com.gct.edge.actuator.controller;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.metadata.CellData;
import com.alibaba.excel.metadata.Head;
import com.alibaba.excel.write.metadata.holder.WriteSheetHolder;
import com.alibaba.excel.write.style.column.AbstractColumnWidthStyleStrategy;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gct.common.core.result.Result;
import com.gct.common.util.StringUtil;
import com.gct.edge.actuator.entity.YieldDaily;
import com.gct.edge.actuator.entity.excel.YieldDailyExcelObj;
import com.gct.edge.actuator.service.YieldDailyService;
import com.gct.edge.actuator.service.YieldService;
import org.apache.poi.ss.usermodel.Cell;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/module/yieldDaily")
public class YieldDailyController {

    @Autowired
    YieldDailyService yieldDailyService;
    @Autowired
    YieldService yieldService;

    @GetMapping("/getWellProdData")
    public Result getWellProdData(@RequestParam String moduleKey, @RequestParam String wellId,
                                  @RequestParam String beginDate, @RequestParam String endDate,
                                  @RequestParam Integer curPage, @RequestParam Integer pageSize) {
        return Result.ok(yieldDailyService.getWellProdData(moduleKey,wellId,beginDate,endDate,curPage,pageSize));
    }

    @GetMapping("/getWellProdGraph")
    public Result getWellProdGraph(@RequestParam String moduleKey, @RequestParam String wellId,
                                   @RequestParam String beginDate, @RequestParam String endDate){
        if (StringUtil.isBlankOrEmpty(moduleKey)) return Result.error("参数条件不足");
        return Result.ok(yieldDailyService.getWellProdGraph(moduleKey,wellId,beginDate,endDate));
    }

    @PostMapping("/exportWellProdData")
    public void exportWellProdData(@RequestBody JSONObject param, HttpServletResponse response) throws IOException{
        String moduleKey = param.getString("moduleKey");
        String wellId  = param.getString("wellId");
        String beginDate = param.getString("beginDate");
        String endDate = param.getString("endDate");
        List<YieldDaily> list = yieldDailyService.getWellProdData(moduleKey, wellId, beginDate, endDate);
        List<YieldDailyExcelObj> collect = list.stream().map(
                x-> {
                    //大转小
                    YieldDailyExcelObj excelObj = ((JSONObject) JSON.toJSON(x)).toJavaObject(YieldDailyExcelObj.class);
                    excelObj.setProTimeStr(excelObj.getProTime().format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
                    return excelObj;
                }
        ).collect(Collectors.toList());
        response.setContentType("application/vnd.ms-excel");
        response.setCharacterEncoding("utf-8");
        EasyExcel.write(response.getOutputStream(), YieldDailyExcelObj.class)
                .registerWriteHandler(new AbstractColumnWidthStyleStrategy() {
                    @Override
                    protected void setColumnWidth(WriteSheetHolder writeSheetHolder, List<CellData> list, Cell cell, Head head, Integer integer, Boolean aBoolean) {
                        writeSheetHolder.getSheet().setColumnWidth(cell.getColumnIndex(),3500);
                    }
                }).sheet("sheet1").doWrite(collect);
    }

    @GetMapping("/calculate")
    public Result calculate(@RequestParam String moduleKey,
                               @RequestParam String wellIdList,
                               @RequestParam String startTime) {
        return yieldDailyService.calculate(moduleKey, wellIdList, startTime);
    }
}
