package com.gct.edge.actuator.entity;

import com.gct.common.util.DateTimeUtil;
import com.gct.common.util.StringUtil;
import com.gct.edge.actuator.exception.ObjectParseException;

import java.lang.reflect.InvocationTargetException;
import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.Objects;

/**
 * @author xusirui 2021/10/26
 * 从gtcvr文件中读取的数据对象类应当实现的转换接口
 */
public interface FileParseObject {

    String SPLIT_STR = "|";

    void initFromTxtRow(String[] objFields) throws ObjectParseException, NoSuchMethodException, InvocationTargetException, IllegalAccessException, InstantiationException;

    default void initFromTxtRow(String[] objFields, String[] colName, String[] colType) throws ObjectParseException, NoSuchMethodException, InvocationTargetException, IllegalAccessException, InstantiationException {
        if (objFields.length != colName.length) {
            throw new ObjectParseException(String.format("预期的此行文件列内容数(%d)与实际(%s)不一致", colName.length, objFields.length));
        }
        for (int i = 0; i < colName.length; i++) {
            if (Objects.isNull(objFields[i]) || StringUtil.isBlankOrEmpty(colName[i])) continue;
            String setMethodName = "set" + colName[i].substring(0, 1).toUpperCase() + colName[i].substring(1);
            switch (colType[i]) {
                case "String":
                    getClass().getMethod(setMethodName, String.class).invoke(this, objFields[i]);
                    break;
                case "Integer":
                    getClass().getMethod(setMethodName, Integer.class).invoke(this, Integer.parseInt(objFields[i]));
                    break;
                case "Double":
                    getClass().getMethod(setMethodName, Double.class).invoke(this, Double.parseDouble(objFields[i]));
                    break;
                case "LocalDateTime":
                    getClass().getMethod(setMethodName, LocalDateTime.class).invoke(this, DateTimeUtil.formatDateTime(objFields[i]));
                    break;
                default:
                    throw new ObjectParseException(String.format("未定义的解析类型%s，请检查colType数组与YieldRealTime类成员定义是否一致", colType[i]));
            }
        }
    }
}
