package com.gct.edge.actuator.entity;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.time.LocalDateTime;

/**
 * @Author: Jyx
 * @Since: 2021/1/28 17:52
 */
@Data
@ApiModel(value = "AoidDailyYield对象", description = "量液日产量存储数据")
@TableName("aoid_daily_yield")
public class YieldDaily {

    @Component
    public static class DoubleSerialize extends JsonSerializer {
        @Override
        public void serialize(Object o, JsonGenerator jsonGenerator, SerializerProvider serializerProvider) throws IOException {
            if (o != null && o instanceof Double) {
                jsonGenerator.writeNumber(Math.round((Double) o * 100) / 100.0);
            }
        }
    }

    @ApiModelProperty(value = "分库分表所需key")
    private String moduleKey;

    @ApiModelProperty(value = "实时时间")
    @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "yyyy-MM-dd", timezone = "GMT+8")
    private LocalDateTime proTime;

    @ApiModelProperty(value = "井号")
    private String wellCommonName;

    @ApiModelProperty(value = "井Id")
    private String wellId;

    @ApiModelProperty(value = "日期")
    @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "yyyy-MM-dd", timezone = "GMT+8")
    private LocalDateTime prodDate;

    //@JsonSerialize(using = DoubleSerialize.class)

    @ApiModelProperty(value = "日产液量 t(实际产量)")
    private Double liqProdDaily;

    //@JsonSerialize(using = DoubleSerialize.class)

    @ApiModelProperty(value = "日产油量 t")
    private Double oilProdDaily;

    //@JsonSerialize(using = DoubleSerialize.class)

    @ApiModelProperty(value = "日产水量 t")
    private Double waterProdDaily;

//    @JsonSerialize(using = DoubleSerialize.class)

    @ApiModelProperty(value = "日产气量 m^3")
    private Double gasProdDaily;

//    @JsonSerialize(using = DoubleSerialize.class)

    @ApiModelProperty(value = "泵效")
    private Double bx;

    @ApiModelProperty(value = "理论排量")
    private Double llpl;

    @ApiModelProperty(value = "生产时率")
    private Integer prodTime;

    @ApiModelProperty(value = "产量系数")
    private Double clxs;

    @JsonSerialize(using = DoubleSerialize.class)
    @ApiModelProperty(value = "冲程")
    private Double stroke;

    @JsonSerialize(using = DoubleSerialize.class)
    @ApiModelProperty(value = "冲次")
    private Double frequence;


    @ApiModelProperty(value = "含水率 %")
    private Double waterCut;


    @ApiModelProperty(value = "泵径")
    private Double bj;


    @ApiModelProperty(value = "泵深")
    private Double bs;

    @ApiModelProperty(value = "沉没度")
    private Double cmd;

    @ApiModelProperty(value = "油嘴直径")
    private Double yz;

    @ApiModelProperty(value = "生产时间")
    private Double scsj;

    @ApiModelProperty(value = "油压")
    private Double yy;

    @ApiModelProperty(value = "回压")
    private Double hy;

    @ApiModelProperty(value = "套压")
    private Double ty;

    @ApiModelProperty(value = "备注")
    private String bz;

    @ApiModelProperty(value = "是否异常")
    private Double sfyc;

    @ApiModelProperty(value = "动液面")
    private Double dym;

    @ApiModelProperty(value = "日志动液面")
    private Double a2Dym;

    @ApiModelProperty(value = "24小时产液量")
    private Double rcy2;

    private Double rcu2;

    private Double rcs2;

    @ApiModelProperty(value = "日志产液量")
    private Double a2LiqProdDaily;

    @ApiModelProperty(value = "日志产油量")
    private Double a2OilProdDaily;

    @ApiModelProperty(value = "日志产水量")
    private Double a2WaterProdDaily;

    @ApiModelProperty(value = "计量产液量")
    private Double testLiquVolDaily;

    private Double a2GasProdDaily;

    private Integer a2ProdTime;

    @ApiModelProperty(value = "有效功图数")
    private Integer yxgts;

    @ApiModelProperty(value = "错误功图数")
    private Integer errorgts;

    private String zdjg;

    @ApiModelProperty(value = "最小电压")
    private Double djdy;

    @ApiModelProperty(value = "作业区")
    private String zyq;

    @ApiModelProperty(value = "计量站")
    private String jlz;

    @ApiModelProperty(value = "平均环境温度")
    private Double hjwd;

    @ApiModelProperty(value = "仪表电流")
    private Double ybdl;

    @ApiModelProperty(value = "设备号")
    private String sbh;

    @ApiModelProperty(value = "信号强度")
    private Double xhqd;

    @ApiModelProperty(value = "压力仪表的今日压力平均值")
    private Double yl;

    @ApiModelProperty(value = "日志备注")
    private String rzbz;

    @ApiModelProperty(value = "水密度")
    private Double smd;

    @ApiModelProperty(value = "油密度")
    private Double ymd;

    @ApiModelProperty(value = "气油比")
    private Double qyb;




    @TableField(exist = false)
    private String jq;

    @TableField(exist = false)
    private String zd;





}
