package com.gct.edge.actuator.handler;

import org.apache.ibatis.type.BaseTypeHandler;
import org.apache.ibatis.type.JdbcType;
import org.apache.ibatis.type.MappedJdbcTypes;
import org.apache.ibatis.type.MappedTypes;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.sql.CallableStatement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;

/**
 * class name: LocalDateTypeHandler
 *
 * @author gxt
 * @version 1.0
 * @since 2021/10/12 下午3:45 周二
 * 重写 mybatis 的 LocalDateTime 时间处理类,不然会出现 getObject result type 异常 两个都需要重写
 */
@Component
@MappedTypes(LocalDateTime.class)
@MappedJdbcTypes(value = JdbcType.TIMESTAMP, includeNullJdbcType = true)
public class LocalDateTimeTypeHandler extends BaseTypeHandler<LocalDateTime>{

    @Override
    public void setNonNullParameter(PreparedStatement ps, int i, LocalDateTime parameter, JdbcType jdbcType) throws SQLException {
        if (parameter!=null) ps.setObject(i, parameter);
    }

    @Override
    public LocalDateTime getNullableResult(ResultSet rs, String columnName) throws SQLException {
        if (StringUtils.isEmpty(rs.getObject(columnName))) {
            return null;
        }
        String[] split = rs.getObject(columnName).toString().split("\\.");
        return LocalDateTime.parse(split[0], DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
    }

    @Override
    public LocalDateTime getNullableResult(ResultSet rs, int columnIndex) throws SQLException {
        if (StringUtils.isEmpty(rs.getObject(columnIndex))) {
            return null;
        }
        return LocalDateTime.parse(rs.getObject(columnIndex).toString(), DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
    }

    @Override
    public LocalDateTime getNullableResult(CallableStatement cs, int columnIndex) throws SQLException {
        if (StringUtils.isEmpty(cs.getObject(columnIndex))) {
            return null;
        }
        return LocalDateTime.parse(cs.getObject(columnIndex).toString(), DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
    }

}
