package com.gct.edge.actuator.remoteDataStruct;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

public class SqlGenerate {

    public static String getLastString(String name){
        String[] split = name.split("\\.");
        return split[split.length-1];
    }

    public static String typeParse(String type) {
        switch (type) {
            case "double":
            case "float":
                return "real";
            case "String":
                return "text";
            case "int":
            case "long":
                return "integer";
            default:
                return "unknown type";
        }
    }



    public static String getClassCreateSql(Class clz){
        Field[] fields = clz.getDeclaredFields();
        StringBuilder sb = new StringBuilder();

        List<IndexObject> uniqueList = new ArrayList<>();
        sb.append("create table if not exists ").append(getLastString(clz.getName())).append("( \n");
        for (int i = 0; i < fields.length; i++) {
            sb.append(fields[i].getName()).append(" ").append(typeParse(getLastString(fields[i].getType().getName())));
            if (fields[i].getAnnotation(PrimaryKey.class)!=null)sb.append(" primary key");
            Index index = fields[i].getAnnotation(Index.class);
            if (index !=null){
                uniqueList.add(new IndexObject(fields[i].getName(),index.value()));
            }
            if(i!=fields.length-1)sb.append(",\n");
        }
        sb.append(");\n\n");
        if (!uniqueList.isEmpty()) {
            sb.append(getIndexCreateSql(clz, uniqueList));
        }
        return sb.toString();
    }

    public static String getIndexCreateSql(Class clz,List<IndexObject> uniqueList){
        uniqueList.sort(Comparator.comparing(IndexObject::getOrder));
        StringBuilder sb = new StringBuilder();
        sb.append("create unique index if not exists ").append(getLastString(clz.getName()));
        uniqueList.forEach(x->sb.append("_").append(x.getColName()));
        sb.append("_uindex on ").append(getLastString(clz.getName())).append(" (");
        uniqueList.forEach(x->sb.append(x.getColName()).append(","));
        sb.deleteCharAt(sb.length()-1); // remove last ','
        sb.append(");\n\n");
        return sb.toString();
    }
    
    

    public static void main(String[] args) throws IOException {

        File file = new File("/home/gxt/IdeaProjects/edge_module_actuator/table_create.sql");
        if (!file.exists())file.createNewFile();
        BufferedWriter bw = new BufferedWriter(new FileWriter(file));
        bw.write(getClassCreateSql(aoid_config_data.class));
        /*bw.write(getClassCreateSql(aoid_daily_yield.class));
        bw.write(getClassCreateSql(aoid_gk_model.class));
        bw.write(getClassCreateSql(aoid_singlegt_yield.class));
        bw.write(getClassCreateSql(aoid_singlewell_config.class));
        bw.write(getClassCreateSql(dgns_gtbaseparam.class));
        bw.write(getClassCreateSql(dgns_gtcard_lib.class));
        bw.write(getClassCreateSql(dgns_gtcvrparam.class));
        bw.write(getClassCreateSql(dgns_gtcvrresult.class));
        bw.write(getClassCreateSql(dgns_gtgktype.class));
        bw.write(getClassCreateSql(dgns_gtmloutcode.class));
        bw.write(getClassCreateSql(dgns_gtserverrunlog.class));
        bw.write(getClassCreateSql(dgns_gtstandard.class));*/
        bw.close();
    }
}

class IndexObject {
    public String colName;
    public Integer order;

    public IndexObject(String colName, Integer order) {
        this.colName = colName;
        this.order = order;
    }

    public String getColName() {
        return colName;
    }

    public Integer getOrder() {
        return order;
    }
}
