package com.gct.edge.actuator.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gct.common.core.result.Result;
import com.gct.common.util.DateTimeUtil;
import com.gct.edge.actuator.entity.AoidConfigData;
import com.gct.edge.actuator.feign.ProtalClient;
import com.gct.edge.actuator.mapper.AoidConfigDataMapper;
import com.gct.edge.actuator.service.AoidConfigDataService;
import com.gct.edge.actuator.service.ServicePlusImpl;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import com.gct.edge.actuator.service.SyncRemoteConfig;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.CopyOnWriteArrayList;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * class name: aoidConfigDataServiceImpl
 *
 * @author gxt
 * @version 1.0
 * @since 2021/12/1 下午2:37 周三
 */
@Service
public class AoidConfigDataServiceImpl extends ServicePlusImpl<AoidConfigDataMapper, AoidConfigData> implements AoidConfigDataService, SyncRemoteConfig {

    @Autowired
    ProtalClient protalClient;

    @Override
    public List<AoidConfigData> getAoidConfigDataList(String moduleKey, String proTime) {
        QueryWrapper<AoidConfigData> wrapper = new QueryWrapper<AoidConfigData>()
                .eq("module_key",moduleKey)
                .eq("pro_time",DateTimeUtil.formatDateTime(proTime));
        return selectMapToObject(wrapper);
    }

    @Override
    public boolean addAoidConfigData(AoidConfigData aoidConfigData) {
        QueryWrapper<AoidConfigData> wrapper = new QueryWrapper<AoidConfigData>()
                .eq("module_key",aoidConfigData.getModuleKey())
                .eq("pro_time",aoidConfigData.getProTime())
                .eq("jh",aoidConfigData.getJh());
        List<AoidConfigData> aoidConfigData1 = baseMapper.selectList(wrapper);
        if (!aoidConfigData1.isEmpty()){
          return baseMapper.update(aoidConfigData,wrapper)>0;
        }
       return baseMapper.insert(aoidConfigData)>0;
    }


    @Override
    public boolean addAoidConfigDataWithList(AoidConfigData aoidConfigData) {
        return baseMapper.insert(aoidConfigData)>0;
    }
    @Override
    public boolean editAoidConfigData(AoidConfigData aoidConfigData) {
        QueryWrapper<AoidConfigData> wrapper =  new QueryWrapper<>();;
            wrapper.eq("module_key", aoidConfigData.getModuleKey())
                    .eq("pro_time", aoidConfigData.getProTime())
                    .eq("jh", aoidConfigData.getJh());
        return baseMapper.update(aoidConfigData, wrapper)>0;
    }

    @Override
    public boolean deleteAoidConfigData(String moduleKey, String proTime) {
        QueryWrapper<AoidConfigData> wrapper = new QueryWrapper<AoidConfigData>()
                .eq("module_key",moduleKey)
                .eq("pro_time",DateTimeUtil.formatDateTime(proTime));
        int delete = baseMapper.delete(wrapper);
        if (delete > 0){
            String s = protalClient.deleteEdgeReleaseConfigMonitorWithModuleKey(moduleKey, proTime);
            Result result = JSONObject.parseObject(s, Result.class);
            return result.getData() == "true";
        }
        return false;
    }

    @Override
    public boolean copyAoidConfigData(Map<String, String> sourceModule, List<String> targetModule){
        List<AoidConfigData> list = baseMapper.selectList(new QueryWrapper<AoidConfigData>().eq("module_key", sourceModule.get("moduleKey"))
                .eq("pro_time", DateTimeUtil.formatDateTime(sourceModule.get("proTime"))));
        AoidConfigData targetAoidConfigData;
        Collection<AoidConfigData> aoidConfigDataList = new CopyOnWriteArrayList<>();
        try {
            for (AoidConfigData aoidConfigData : list) {
                for (String s : targetModule) {
                    targetAoidConfigData = new AoidConfigData();
                    BeanUtils.copyProperties(aoidConfigData,targetAoidConfigData);
                    targetAoidConfigData.setModuleKey(s);
                    aoidConfigDataList.add(targetAoidConfigData);
                }
            }
        }catch (Exception e){
            e.printStackTrace();
        }
        return this.saveBatch(aoidConfigDataList);
    }

    @Override
    public List<String> getData(String moduleKey, LocalDateTime proTime) {
         return this.list(new QueryWrapper<AoidConfigData>()
                 .eq("module_key",moduleKey)
                 .eq("pro_time",proTime))
                 .stream()
                 .map(AoidConfigData::toFileRowString)
                 .collect(Collectors.toList());
    }
}
