package com.gct.edge.actuator.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gct.common.util.DateTimeUtil;
import com.gct.edge.actuator.entity.AoidConfigData;
import com.gct.edge.actuator.entity.AoidGkModel;
import com.gct.edge.actuator.entity.AoidSinglewellConfig;
import com.gct.edge.actuator.entity.DgnsGtcardLib;
import com.gct.edge.actuator.mapper.AoidGkModelMapper;
import com.gct.edge.actuator.service.AoidGkModelService;
import com.gct.edge.actuator.service.DgnsGtcardLibService;
import com.gct.edge.actuator.service.ServicePlusImpl;
import com.gct.edge.actuator.service.SyncRemoteConfig;
import com.gct.edge.base.entity.TableNameEnum;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.xml.ws.Action;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * class name: AoidGkModelServiceImpl
 *
 * @author gxt
 * @version 1.0
 * @since 2021/12/13 下午2:28 周一
 */
@Service
public class AoidGkModelServiceImpl extends ServicePlusImpl<AoidGkModelMapper, AoidGkModel> implements AoidGkModelService, SyncRemoteConfig {


    @Autowired
    DgnsGtcardLibService dgnsGtcardLibService;

    @Override
    public Page<AoidGkModel> getModelDataListPage(String modulekey,int pageSize, int pageNum) {
        Page<AoidGkModel> aoidGkModelPage = baseMapper.selectPage(
                new Page<>(pageNum, pageSize),
                new LambdaQueryWrapper<AoidGkModel>()
                .eq(AoidGkModel::getModuleKey,modulekey)
                .orderByAsc(AoidGkModel::getResult));
        Map<String, Integer> dgnsCardLibGTNumWithResult = dgnsGtcardLibService.getDgnsCardLibGTNumWithResult(modulekey);
        for (AoidGkModel record : aoidGkModelPage.getRecords()) {
            for (String s : dgnsCardLibGTNumWithResult.keySet()) {
                if (s.equals(record.getResult())) record.setGtNum(dgnsCardLibGTNumWithResult.get(s));
            }
        }
        return aoidGkModelPage;
    }

    @Override
    public boolean saveAoidGkModelLib(JSONObject jsonObject) {
        JSONObject modelLib = jsonObject.getJSONObject("modelLib");
        String moduleKey = jsonObject.getString("moduleKey");
        LocalDateTime now = LocalDateTime.now();
        String s = DateTimeUtil.formatString(now);
        int count = 0;
        if (!StringUtils.isEmpty(modelLib.getString("data"))){
            String[]  split = modelLib.getString("data").trim().split("\n");
            for (String str : split) {
                String[] split1 = str.split("\\|");
                QueryWrapper<AoidGkModel> wrapper = new QueryWrapper<AoidGkModel>()
                        .eq("module_key",moduleKey)
                        .eq("result",split1[1]);
                List<AoidGkModel> aoidGkModels = baseMapper.selectList(wrapper);
                AoidGkModel aoidGkModel = new AoidGkModel();
                aoidGkModel.setProTime(DateTimeUtil.formatDateTime(s));
                aoidGkModel.setModuleKey(moduleKey);
                aoidGkModel.setResult(split1[1]);
                aoidGkModel.setGkxs(Double.parseDouble(split1[0]));
                if (aoidGkModels.isEmpty()){
                   count = baseMapper.insert(aoidGkModel)+count;
                }else {
                    count = baseMapper.update(aoidGkModel, wrapper) +count;
                }
            }
            return count == split.length;
        }
        return false;
    }

    @Override
    public boolean editAoidGKModelLib(JSONObject jsonObject) {
        JSONObject modelLib = jsonObject.getJSONObject("modelLib");
        String moduleKey = jsonObject.getString("moduleKey");
        AoidGkModel aoidGkModel = new AoidGkModel();
        aoidGkModel.setGkxs(Double.parseDouble(String.valueOf(modelLib.get("gkxs"))));
        QueryWrapper<AoidGkModel> wrapper = new  QueryWrapper<AoidGkModel>()
                .eq("module_key",moduleKey)
                .eq("result",modelLib.getString("result"))
                .eq("pro_time",DateTimeUtil.formatDateTime(modelLib.getString("proTime").replace("T"," ")));
        return baseMapper.update(aoidGkModel,wrapper) > 0 ;
    }

    @Override
    public boolean copyAoidGkModelLib(JSONObject jsonObject) {
        String sourceModule = jsonObject.getString("sourceModule");
        JSONArray targetModule = jsonObject.getJSONArray("targetModule");
        QueryWrapper<AoidGkModel> wrapper =new QueryWrapper<AoidGkModel>()
                .eq("module_key",sourceModule);
        List<AoidGkModel> aoidGkModels = baseMapper.selectList(wrapper);
        int insert = 0;
        int delete = 0 ;
        for (int i = 0; i < targetModule.size(); i++) {
            for (AoidGkModel aoidGkModel : aoidGkModels) {
                aoidGkModel.setModuleKey(targetModule.getString(i));
                delete += deleteAoidGkModel(aoidGkModel.getModuleKey(),aoidGkModel.getResult());
            }
            if (delete == aoidGkModels.size()) {
                if (this.saveBatch(aoidGkModels,aoidGkModels.size())){
                    insert +=  aoidGkModels.size();
                }
            }
        }
        return insert == (aoidGkModels.size()*targetModule.size());
    }

    private int deleteAoidGkModel(String moduleKey,String result){
         QueryWrapper<AoidGkModel> wrapper =new QueryWrapper<AoidGkModel>()
                    .eq("module_key",moduleKey)
                    .eq("result",result);
        return baseMapper.delete(wrapper);
    }

    @Override
    public boolean deleteAoidGkModelLib(JSONObject jsonObject) {
        String moduleKey = jsonObject.getString("moduleKey");
        QueryWrapper<AoidGkModel> wrapper = new QueryWrapper<AoidGkModel>()
                .eq("module_key",moduleKey)
                .eq("result", jsonObject.getString("result"))
                .eq("pro_time",DateTimeUtil.formatDateTime(jsonObject.getString("proTime").replace("T"," ")))
                .eq("gkxs",Double.parseDouble(String.valueOf(jsonObject.get("gkxs"))));
        if (dgnsGtcardLibService.deleteCardLibByResult(moduleKey,jsonObject.getString("result"))){
           return baseMapper.delete(wrapper) > 0;
        }
        return false;
    }

    @Override
    public int editAoidGkModelLib(Map<String, String> map) {
        int count = 0;
        QueryWrapper<AoidGkModel> wrapper = new QueryWrapper<AoidGkModel>()
                .eq("module_key",map.get("moduleKey"))
                .eq("result",map.get("result"));
        AoidGkModel aoidGkModel = new AoidGkModel();
        AoidGkModel aoidGkModel1 = baseMapper.selectOne(wrapper);
        LocalDateTime now = LocalDateTime.now();
        String s = DateTimeUtil.formatString(now);
        if (StringUtils.isEmpty(aoidGkModel1)){
            aoidGkModel.setResult(map.get("result"));
            aoidGkModel.setGkxs(1.0);
            aoidGkModel.setModuleKey(map.get("moduleKey"));
            aoidGkModel.setProTime(DateTimeUtil.formatDateTime(s));
            count += baseMapper.insert(aoidGkModel);
        }else {
            count ++;
        }
        return count;
    }

    @Override
    public List<String> downloadModelInfo(String modulekey) {
        List<AoidGkModel> aoidGkModels = baseMapper.selectList(
                new LambdaQueryWrapper<AoidGkModel>()
                        .eq(AoidGkModel::getModuleKey, modulekey)
                        .orderByAsc(AoidGkModel::getResult));
        Map<String, Integer> dgnsCardLibGTNumWithResult = dgnsGtcardLibService.getDgnsCardLibGTNumWithResult(modulekey);
        for (AoidGkModel aoidGkModel : aoidGkModels) {
            for (String s : dgnsCardLibGTNumWithResult.keySet()) {
                if (s.equals(aoidGkModel.getResult())) aoidGkModel.setGtNum(dgnsCardLibGTNumWithResult.get(s));
            }
        }
        return aoidGkModels.stream().map(AoidGkModel::toFileRowString).collect(Collectors.toList());
    }

    @Override
    public List<String> getData(String moduleKey, LocalDateTime proTime) {
        return this.list(new QueryWrapper<AoidGkModel>()
                        .eq("module_key", moduleKey))
//                        .eq("pro_time", proTime)
                .stream()
                .map(AoidGkModel::toFileRowString)
                .collect(Collectors.toList());
    }
}
