package com.gct.edge.actuator.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.gct.common.core.result.Result;
import com.gct.common.util.DateTimeUtil;
import com.gct.common.util.StringUtil;
import com.gct.edge.actuator.entity.AoidProdState;
import com.gct.edge.actuator.entity.YieldRealTime;
import com.gct.edge.actuator.mapper.AoidProdStateMapper;
import com.gct.edge.actuator.mapper.YieldRealTimeMapper;
import com.gct.edge.actuator.service.AoidProdStateService;
import com.gct.edge.actuator.service.ServicePlusImpl;
import com.gct.edge.actuator.util.DataTranslator;
import com.gct.edge.base.entity.TableNameEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.InputStream;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;

/**
 * @author lloyd 2021/10/26
 */
@Service
public class AoidProdStateServiceImpl extends ServicePlusImpl<AoidProdStateMapper, AoidProdState> implements AoidProdStateService {

    @Autowired
    public AoidProdStateMapper aoidProdStateMapper;

    @Override
    public Result loadDataFromFile(InputStream file, String moduleKey, TableNameEnum tableKey) {
        boolean checkParams = !StringUtil.isBlankOrEmpty(moduleKey);
        if (Objects.isNull(tableKey)) checkParams = false;
        if (!checkParams) return Result.error("input params is not supported");

        List<AoidProdState> data = DataTranslator.transFileToFileParseObject(file, AoidProdState.class);
        data.forEach(x -> {
            x.setModuleKey(moduleKey);
        });
        int sus = saveOnConflictDoNothingBatch(data);
        return sus > 0 ? Result.ok("导入了" + (sus) + "条") : Result.error("导入了0条,检查格式、重复？");
    }

    @Override
    public List<AoidProdState> getAoidProdStateList(String moduleKey, String wellId, String startTime) {
        LocalDateTime start = DateTimeUtil.formatDateTime(startTime + " 00:00:00");
        LocalDateTime end = DateTimeUtil.formatDateTime(startTime + " 23:59:59");

        LambdaQueryWrapper<AoidProdState> queryWrapper = new LambdaQueryWrapper<AoidProdState>()
                .eq(AoidProdState::getModuleKey, moduleKey)
                .between(AoidProdState::getProTime, start, end)
                .eq(AoidProdState::getWellId, wellId)
                .orderByAsc(AoidProdState::getProTime);
        List<AoidProdState> maps = aoidProdStateMapper.selectList(queryWrapper);

        return maps;
    }
}
