package com.gct.edge.actuator.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.gct.common.util.DateTimeUtil;
import com.gct.edge.actuator.entity.AoidSinglewellConfig;
import com.gct.edge.actuator.mapper.AoidSinglewellConfigMapper;
import com.gct.edge.actuator.service.AoidSinglewellConfigService;
import com.gct.edge.actuator.service.ServicePlusImpl;
import com.gct.edge.actuator.service.SyncRemoteConfig;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class AoidSinglewellConfigServiceImpl extends ServicePlusImpl<AoidSinglewellConfigMapper, AoidSinglewellConfig> implements AoidSinglewellConfigService, SyncRemoteConfig {
    @Autowired
    private AoidSinglewellConfigMapper aoidSingleWellConfigMapper;

    @Autowired
    private AoidSinglewellConfigService aoidSinglewellConfigService;

    @Override
    public List<AoidSinglewellConfig> getSingleWellConfigs(String moduleKey, String releaseVersion) {
        if ("".equals(moduleKey) || "".equals(releaseVersion)) return null;
        QueryWrapper<AoidSinglewellConfig> last = new QueryWrapper<AoidSinglewellConfig>()
                .eq("module_key", moduleKey).eq("pro_time", DateTimeUtil.formatDateTime(releaseVersion))
                .last(" order by id asc ");
        List<AoidSinglewellConfig> aoidSinglewellConfigs1 = baseMapper.selectList(last);
        return aoidSinglewellConfigs1;
    }

    @Override
    public String insertSingleWellConfig(JSONObject jsonObject) {
        String resultString = "";
        jsonObject.remove("flag");
        AoidSinglewellConfig aoidSinglewellConfig = JSON.toJavaObject(jsonObject, AoidSinglewellConfig.class);
        LocalDateTime localDateTime = DateTimeUtil.formatDateTime(jsonObject.getString("proTime").replace("T", " "));

        aoidSinglewellConfig.setProTime(localDateTime);
        int insert = baseMapper.insert(aoidSinglewellConfig);
        resultString = insert == 1 ? "success" : "error";
        if ("insetCopyList".equals(jsonObject.getString("insetCopyList"))) {
            QueryWrapper<AoidSinglewellConfig> aoidSinglewellConfigQueryWrapper = new QueryWrapper<>();
            aoidSinglewellConfigQueryWrapper.eq("module_key", aoidSinglewellConfig.getModuleKey()).eq("pro_time", DateTimeUtil.formatDateTime(jsonObject.getString("releaseVersion").replace("T", " ")));
            List<AoidSinglewellConfig> aoidSinglewellConfigs = baseMapper.selectList(aoidSinglewellConfigQueryWrapper);
           /* List<AoidSinglewellConfig> collect = aoidSinglewellConfigs.stream().map(
                    s -> {
                        int maxId = getMaxId(jsonObject.getString("moduleKey"));
                        s.setProTime(localDateTime);
                        s.setId(maxId);
                        return s;
                    }
            ).collect(Collectors.toList());*/
            for (int i = 0; i < aoidSinglewellConfigs.size(); i++) {
                aoidSinglewellConfigs.get(i).setProTime(localDateTime);
            }
            boolean b = aoidSinglewellConfigService.saveBatch(aoidSinglewellConfigs);
            resultString = b == true ? "success" : "error";
        }
        return resultString;

    }

    @Override
    public String updateSingleWellConfig(JSONObject jsonObject) {
        AoidSinglewellConfig aoidSinglewellConfig = JSON.toJavaObject(jsonObject, AoidSinglewellConfig.class);
        int update = baseMapper.insert(aoidSinglewellConfig);
        return update == 1 ? "success" : "error";
    }

    @Override
    public String editSingleWellConfig(JSONObject jsonObject) {

        String resultString = "";
        String release_version = jsonObject.getString("release_version");
        Integer id = Integer.valueOf(jsonObject.getString("oldId"));
        LocalDateTime localDateTime = DateTimeUtil.formatDateTime(release_version);
        AoidSinglewellConfig aoidSinglewellConfig = JSON.toJavaObject(jsonObject, AoidSinglewellConfig.class);

        if (Integer.valueOf(jsonObject.getString("release_status")) == 2) {

            aoidSinglewellConfig.setProTime(localDateTime);
            QueryWrapper<AoidSinglewellConfig> aoidSinglewellConfigQueryWrapper = new QueryWrapper<>();
            aoidSinglewellConfigQueryWrapper
                    .eq("module_key", aoidSinglewellConfig.getModuleKey())
                    .eq("pro_time", DateTimeUtil.formatDateTime(jsonObject.getString("proTime").replace("T", " ")))
                    .last(" and  id  != " + id);
            List<AoidSinglewellConfig> aoidSinglewellConfigs = baseMapper.selectList(aoidSinglewellConfigQueryWrapper);
            for (int i = 0; i < aoidSinglewellConfigs.size(); i++) {
                aoidSinglewellConfigs.get(i).setProTime(localDateTime);
            }
            aoidSinglewellConfigs.add(aoidSinglewellConfig);
            boolean b = aoidSinglewellConfigService.saveBatch(aoidSinglewellConfigs);
            resultString = b == true ? "success" : "error";
        } else {
            UpdateWrapper<AoidSinglewellConfig> updateWrapper = new UpdateWrapper<>();
            updateWrapper
                    .set("id", aoidSinglewellConfig.getId())
                    .eq("id", id)
                    .eq("module_key", aoidSinglewellConfig.getModuleKey())
                    .eq("pro_time", aoidSinglewellConfig.getProTime());

            int b = baseMapper.update(aoidSinglewellConfig, updateWrapper);
            resultString = b == 1 ? "success" : "修改失败";
        }
        return resultString;
    }

    @Override
    public String deleteSingleWellConfig(String moduleKey, String proTime) {
        QueryWrapper<AoidSinglewellConfig> aoidSinglewellConfigQueryWrapper = new QueryWrapper<>();
        aoidSinglewellConfigQueryWrapper.eq("module_key", moduleKey).eq("pro_time", DateTimeUtil.formatDateTime(proTime.replace("T", " ")));
        return baseMapper.delete(aoidSinglewellConfigQueryWrapper) == 1 ? "success" : "error";
    }

    @Override
    public Map<String, Object> copySingleWellConfig(JSONObject jsonObject) {
        String moduleKey = jsonObject.getString("moduleKey");
        String targetModuleKey = jsonObject.getString("targetModuleKey");
        String release_version = jsonObject.getString("relsease_version");
        LocalDateTime localDateTime = DateTimeUtil.formatDateTime(jsonObject.getString("releaseVersion").replace("T", " "));
        QueryWrapper<AoidSinglewellConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("module_key", moduleKey)
                .eq("pro_time", localDateTime);
        List<AoidSinglewellConfig> gtcvrRuntimeConfigs = baseMapper.selectList(queryWrapper);
        for (int i = 0; i < gtcvrRuntimeConfigs.size(); i++) {
            gtcvrRuntimeConfigs.get(i).setModuleKey(targetModuleKey);
            gtcvrRuntimeConfigs.get(i).setProTime(DateTimeUtil.formatDateTime(release_version.replace("T", " ")));

        }
        boolean b = this.saveBatch(gtcvrRuntimeConfigs);
        return null;
    }

    @Override
    public String deleteSingleWellConfig(JSONObject jsonObject) {
        Integer flag = Integer.valueOf(jsonObject.getString("status"));
        String release_version = jsonObject.getString("release_version");
        LocalDateTime localDateTime = DateTimeUtil.formatDateTime(jsonObject.getString("proTime").replace("T", " "));
        String resultString = "";
        if (flag == 0) {
            QueryWrapper<AoidSinglewellConfig> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("module_key", jsonObject.getString("moduleKey"))
                    .eq("id", Integer.valueOf(jsonObject.getString("id")))
                    .eq("pro_time", localDateTime);
            int delete = baseMapper.delete(queryWrapper);
            resultString = delete >= 1 ? "success" : "error";
        } else {
            QueryWrapper<AoidSinglewellConfig> aoidSinglewellConfigQueryWrapper = new QueryWrapper<>();
            aoidSinglewellConfigQueryWrapper
                    .eq("module_key", jsonObject.getString("moduleKey"))
                    .eq("pro_time", localDateTime)
                    .last(" and  id  != " + jsonObject.getString("id"));
            List<AoidSinglewellConfig> aoidSinglewellConfigs = baseMapper.selectList(aoidSinglewellConfigQueryWrapper);
            for (int i = 0; i < aoidSinglewellConfigs.size(); i++) {
                aoidSinglewellConfigs.get(i).setProTime(DateTimeUtil.formatDateTime(release_version));
            }
            if (aoidSinglewellConfigs.size() == 0) {
                return "success";
            }
            boolean b = this.saveBatch(aoidSinglewellConfigs);
            resultString = b == true ? "success" : "error";
        }
        return resultString;
    }

    @Override
    public String judgeIdExists(Integer id, String moduleKey, String releaseVersion) {
        QueryWrapper<AoidSinglewellConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("id", id).eq("module_key", moduleKey).le("pro_time", DateTimeUtil.formatDateTime(releaseVersion));
        List<AoidSinglewellConfig> aoidSinglewellConfigs = baseMapper.selectList(queryWrapper);
        if (aoidSinglewellConfigs.size() > 0) return "id: " + id + " 已存在，请重新输入";
        else return "success";
    }

    @Override
    public List<String> getData(String moduleKey, LocalDateTime proTime) {
        return this.list(new QueryWrapper<AoidSinglewellConfig>()
                        .eq("module_key", moduleKey)
                        .eq("pro_time", proTime))
                .stream()
                .map(AoidSinglewellConfig::toFileRowString)
                .collect(Collectors.toList());
    }


    public int getMaxId(String moduleKey) {
        LocalDateTime now = LocalDateTime.now();
        String s = DateTimeUtil.formatString(now);
        QueryWrapper<AoidSinglewellConfig> aoidSinglewellConfigQueryWrapper = new QueryWrapper<>();
        aoidSinglewellConfigQueryWrapper.eq("module_key", moduleKey)
                .le("pro_time", now.plusDays(1))
                .last("order by id desc limit 1 ").select("id");
        List<AoidSinglewellConfig> aoidSinglewellConfigs = baseMapper.selectList(aoidSinglewellConfigQueryWrapper);
        if (aoidSinglewellConfigs.size() == 0)
            return 1;
        else
            return aoidSinglewellConfigs.get(0).getId() + 1;
    }
}
