package com.gct.edge.actuator.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gct.common.util.DateTimeUtil;
import com.gct.common.util.SGTUtil;
import com.gct.edge.actuator.entity.AoidConfigData;
import com.gct.edge.actuator.entity.DgnsGtcardLib;
import com.gct.edge.actuator.entity.DiagnoseResultRealTime;
import com.gct.edge.actuator.mapper.DgnsGtcardLibMapper;
import com.gct.edge.actuator.service.*;
import com.gct.edge.actuator.service.AoidGkModelService;
import com.gct.edge.actuator.service.DgnsGtcardLibService;
import com.gct.edge.actuator.service.DiagnoseService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gct.edge.actuator.service.ServicePlusImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * class name: DgnsGtcardLibServiceImpl
 *
 * @author gxt
 * @version 1.0
 * @since 2021/12/13 下午2:22 周一
 */
@Service
public class DgnsGtcardLibServiceImpl extends ServicePlusImpl<DgnsGtcardLibMapper, DgnsGtcardLib> implements DgnsGtcardLibService, SyncRemoteConfig {
    @Autowired
    DiagnoseService diagnoseService;
    @Autowired
    AoidGkModelService aoidGkModelService;

    @Override
    public Page<DgnsGtcardLib> getDgnsCardLibListPage(String moduleKey, int pageSize, int pageNum, String result) {
        QueryWrapper<DgnsGtcardLib> wrapper = new QueryWrapper<DgnsGtcardLib>();
        if (!StringUtils.isEmpty(result)) {
            wrapper = wrapper.eq("result", result);
        }
        wrapper.eq("module_key", moduleKey).last("order by pro_time desc");
        Page<DgnsGtcardLib> dgnsGtcardLibPage = baseMapper.selectPage(new Page<>(pageNum, pageSize), wrapper);
        for (DgnsGtcardLib record : dgnsGtcardLibPage.getRecords()) {
            record.setIndicatorDiagram(Arrays.deepToString(SGTUtil.encodeToDoubleArray(record.getIndicatorDiagram())));
        }
        return dgnsGtcardLibPage;
    }

    @Override
    public boolean saveDgnsCardLib(JSONObject jsonObject) {
        if (!StringUtils.isEmpty(jsonObject.get("card"))) {
            DgnsGtcardLib dgnsGtcardLib = JSONObject.parseObject(jsonObject.getString("card"), DgnsGtcardLib.class);
            return baseMapper.updateById(dgnsGtcardLib) > 0;
        } else if (!StringUtils.isEmpty(jsonObject.get("data"))) {
            String moduleKey = jsonObject.getString("moduleKey");
            JSONObject jsonData = jsonObject.getJSONObject("data");
            String[] split = jsonData.getString("data").split("\n");
            List<String> wellIdList = new ArrayList<>();
            List<LocalDateTime> proTimeList = new ArrayList<>();
            for (String s : split) {
                String[] split1 = s.split(",");
                wellIdList.add(split1[0]);
                proTimeList.add(DateTimeUtil.formatDateTime(split1[1]));
            }
            QueryWrapper<DiagnoseResultRealTime> diaWapper = new QueryWrapper<DiagnoseResultRealTime>()
                    .eq("module_key", moduleKey)
                    .in("well_id", wellIdList)
                    .in("pro_time", proTimeList);
            List<DiagnoseResultRealTime> diagnoseResultRealTimes = diagnoseService.getBaseMapper().selectList(diaWapper);
            if (!diagnoseResultRealTimes.isEmpty()) {
                DgnsGtcardLib dgnsGtcardLib;
                int insert = 0;
                Map<String, String> map;
                for (DiagnoseResultRealTime diagnoseResultRealTime : diagnoseResultRealTimes) {
                    dgnsGtcardLib = new DgnsGtcardLib();
                    String[] sgtList = diagnoseResultRealTime.getWorkDiagram().split(",");
                    List<String> list = new ArrayList<>();
                    for (int i = 0; i < sgtList.length; i++) {
                        if (i % 2 != 0) {
                            BigDecimal bigDecimal = new BigDecimal(sgtList[i]).divide(new BigDecimal("100.0"));
                            list.add(bigDecimal.toString());
                        }
                    }
                    String[] loads = list.toArray(new String[0]);
                    if (loads.length > 0) {
                        dgnsGtcardLib.setMaxload(getLoads(loads).getMax());
                        dgnsGtcardLib.setMinload(getLoads(loads).getMin());
                    }
                    dgnsGtcardLib.setProDate(DateTimeUtil.formatString(diagnoseResultRealTime.getProTime()));
                    dgnsGtcardLib.setModuleKey(diagnoseResultRealTime.getModuleKey());
                    dgnsGtcardLib.setIndicatorDiagram(diagnoseResultRealTime.getWorkDiagram());
                    dgnsGtcardLib.setResult(diagnoseResultRealTime.getResult1());
                    dgnsGtcardLib.setWellId(diagnoseResultRealTime.getWellId());

                    map = new HashMap<>();
                    //TODO card 增加时 model 中没有的要增加
                    map.put("moduleKey", diagnoseResultRealTime.getModuleKey());
                    map.put("result", diagnoseResultRealTime.getResult1());
                    if (aoidGkModelService.editAoidGkModelLib(map) >0) {
                        insert += baseMapper.insert(dgnsGtcardLib);
                    }
                }
                return diagnoseResultRealTimes.size() == insert;
            }
            return false;
        }
        return false;
    }

    private DoubleSummaryStatistics getLoads(String[] strings) {
        double[] doubles = new double[strings.length];
        for (int i = 0; i < strings.length; i++) {
            doubles[i] = Double.parseDouble(strings[i]);
        }
        return Arrays.stream(doubles).summaryStatistics();
    }

    @Override
    public boolean editCardLib(JSONObject jsonObject) {
        String moduleKey = jsonObject.getString("moduleKey");
        JSONArray cardLib = jsonObject.getJSONArray("cardLib");
        QueryWrapper<DgnsGtcardLib> wrapper;
        DgnsGtcardLib dgnsGtcardLib;
        Map<String, String> modelMap;
        int update = 0;
        int modelCount = 0;
        List<DgnsGtcardLib> list = new ArrayList<>();
        for (Object o : cardLib) {
            modelMap = new HashMap<>();
            Map<String, Object> map = (Map<String, Object>) o;
            dgnsGtcardLib = new DgnsGtcardLib();
            dgnsGtcardLib.setId(String.valueOf(map.get("id")));
            dgnsGtcardLib.setResult(String.valueOf(map.get("result")));
            modelMap.put("moduleKey", moduleKey);
            //modelMap.put("oldResult",String.valueOf(map.get("oldResult")));
            modelMap.put("result", String.valueOf(map.get("result")));
            list.add(dgnsGtcardLib);
            modelCount += aoidGkModelService.editAoidGkModelLib(modelMap);
        }
        if (modelCount == cardLib.size()) {
            for (DgnsGtcardLib gtcardLib : list) {
                wrapper = new QueryWrapper<DgnsGtcardLib>().eq("module_key", moduleKey).eq("id", gtcardLib.getId());
                update += baseMapper.update(gtcardLib, wrapper);
            }
            return update == cardLib.size();
        }
        return false;
    }

    @Override
    public boolean copyCardLib(JSONObject jsonObject) {
        String sourceModule = jsonObject.getString("sourceModule");
        JSONArray targetModule = jsonObject.getJSONArray("targetModule");
        QueryWrapper<DgnsGtcardLib> wrapper = new QueryWrapper<DgnsGtcardLib>()
                .eq("module_key", sourceModule);
        List<DgnsGtcardLib> dgnsGtcardLibs = baseMapper.selectList(wrapper);
        int insert = 0;
        for (int i = 0; i < targetModule.size(); i++) {
            for (DgnsGtcardLib dgnsGtcardLib : dgnsGtcardLibs) {
                dgnsGtcardLib.setModuleKey(targetModule.getString(i));
            }
            if (this.saveOnConflictDoNothingBatch(dgnsGtcardLibs) >= 0) {
                insert += dgnsGtcardLibs.size();
            }
        }
        return insert == (dgnsGtcardLibs.size() * targetModule.size());
    }

    @Override
    public boolean deleteCardLib(JSONObject jsonObject) {
        QueryWrapper<DgnsGtcardLib> wrapper = new QueryWrapper<DgnsGtcardLib>()
                .eq("module_key", jsonObject.getString("moduleKey"))
                .eq("id", jsonObject.getString("id"));
        return baseMapper.delete(wrapper) > 0;
    }

    @Override
    public boolean deleteCardLibByResult(String moduleKey, String result) {
        QueryWrapper<DgnsGtcardLib> wrapper = new QueryWrapper<DgnsGtcardLib>()
                .eq("module_key", moduleKey).eq("result", result);
        List<DgnsGtcardLib> dgnsGtcardLibs = baseMapper.selectList(wrapper);
        if (!dgnsGtcardLibs.isEmpty()) return baseMapper.delete(wrapper) > 0;
        else return true;
    }

    @Override
    public Map<String, Integer> getDgnsCardLibGTNumWithResult(String moduleKey) {
        QueryWrapper<DgnsGtcardLib> wrapper = new QueryWrapper<DgnsGtcardLib>()
                .eq("module_key", moduleKey);
        List<DgnsGtcardLib> dgnsGtcardLibs = baseMapper.selectList(wrapper);
        Map<String, Integer> resMap = new HashMap<>();
        if (!dgnsGtcardLibs.isEmpty()) {
            Map<String, List<DgnsGtcardLib>> collect = dgnsGtcardLibs.stream().collect(Collectors.groupingBy(DgnsGtcardLib::getResult));
            for (Map.Entry<String, List<DgnsGtcardLib>> stringListEntry : collect.entrySet()) {
                resMap.put(stringListEntry.getKey(), stringListEntry.getValue().size());
            }
        }
        return resMap;
    }

    @Override
    public List<String> downloadCardlibInfo(String moduleKey, String result) {
        QueryWrapper<DgnsGtcardLib> wrapper = new QueryWrapper<DgnsGtcardLib>();
        if (!StringUtils.isEmpty(result)) {
            wrapper = wrapper.eq("result", result);
        }
        wrapper.eq("module_key", moduleKey).last("order by pro_time desc");
        return baseMapper.selectList(wrapper).stream().map(DgnsGtcardLib::toFileRowString).collect(Collectors.toList());
    }

    @Override
    public List<String> getData(String moduleKey, LocalDateTime proTime) {
        return this.list(new QueryWrapper<DgnsGtcardLib>()
                                .eq("module_key", moduleKey)
//                        .eq("pro_time",proTime)
                )
                .stream()
                .map(DgnsGtcardLib::toFileRowString)
                .collect(Collectors.toList());
    }
}
