package com.gct.edge.actuator.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gct.common.util.DateTimeUtil;
import com.gct.edge.actuator.entity.DgnsGtgktype;
import com.gct.edge.actuator.mapper.DgnsGtgktypeMapper;
import com.gct.edge.actuator.service.DgnsGtgktypeService;
import com.gct.edge.actuator.service.SyncRemoteConfig;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author Lijian
 * @since 2022-03-16
 */
@Service
public class DgnsGtgktypeServiceImpl extends ServiceImpl<DgnsGtgktypeMapper, DgnsGtgktype> implements DgnsGtgktypeService, SyncRemoteConfig {

    @Override
    public List<DgnsGtgktype> getDgnsGtgktype(String moduleKey) {

        QueryWrapper<DgnsGtgktype> last = new QueryWrapper<DgnsGtgktype>()
                .eq("module_key", moduleKey).le("pro_time", LocalDateTime.now().plusDays(1))
                .select("pro_time").last("order by pro_time desc limit 1 ");
        List<DgnsGtgktype> dgnsGtgktypes1 = baseMapper.selectList(last);
        if (dgnsGtgktypes1.size() > 0) {
            QueryWrapper<DgnsGtgktype> dgnsGtgktypeQueryWrapper = new QueryWrapper<>();
            dgnsGtgktypeQueryWrapper.eq("module_key", moduleKey)
                    .eq("pro_time", dgnsGtgktypes1.get(0).getProTime());
            List<DgnsGtgktype> dgnsGtgktypes = baseMapper.selectList(dgnsGtgktypeQueryWrapper);
            return dgnsGtgktypes;
        } else {
            return null;
        }

    }

    @Override
    public String addDgnsGtgktype(JSONObject jsonObject) {
        jsonObject.remove("flag");
        jsonObject.put("id", getMaxId(jsonObject.getString("moduleKey")));
        DgnsGtgktype dgnsGtgktype = JSON.toJavaObject(jsonObject, DgnsGtgktype.class);
        dgnsGtgktype.setProTime(DateTimeUtil.formatDateTime(jsonObject.getString("proTime").replace("T", " ")));
        return baseMapper.insert(dgnsGtgktype) == 1 ? "success" : "error";
    }

    @Override
    public String deleteDgnsGtgktype(String moduleKey, String proTime) {
        QueryWrapper<DgnsGtgktype> dgnsGtgktypeQueryWrapper = new QueryWrapper<>();
        dgnsGtgktypeQueryWrapper.eq("module_key", moduleKey).eq("pro_time", DateTimeUtil.formatDateTime(proTime.replace("T", " ")));
        int delete = baseMapper.delete(dgnsGtgktypeQueryWrapper);
        return delete == 1 ? "success" : "error";
    }

    @Override
    public Map<String, Object> copyDgnsGtgktype(JSONObject jsonObject) {

        LocalDateTime now = LocalDateTime.now();
        String moduleKey = jsonObject.getString("moduleKey");
        String targetModuleKey = jsonObject.getString("targetModuleKey");
        String release_version = jsonObject.getString("relsease_version");
        QueryWrapper<DgnsGtgktype> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("module_key", moduleKey)
                .le("pro_time", now.plusDays(1))
                .last("order by pro_time desc limit 1");
        List<DgnsGtgktype> gtcvrRuntimeConfigs = baseMapper.selectList(queryWrapper);
        DgnsGtgktype gtcvrRuntimeConfig = gtcvrRuntimeConfigs.get(0);
        gtcvrRuntimeConfig.setModuleKey(targetModuleKey);
        gtcvrRuntimeConfig.setProTime(DateTimeUtil.formatDateTime(release_version.replace("T", " ")));
        gtcvrRuntimeConfig.setId(getMaxId(targetModuleKey));
        int insert = baseMapper.insert(gtcvrRuntimeConfig);

        return null;
    }

    public int getMaxId(String moduleKey) {
        LocalDateTime now = LocalDateTime.now();
        String s = DateTimeUtil.formatString(now);
        QueryWrapper<DgnsGtgktype> aoidSinglewellConfigQueryWrapper = new QueryWrapper<>();
        aoidSinglewellConfigQueryWrapper.eq("module_key", moduleKey)
                .le("pro_time", now.plusDays(1))
                .last("order by id desc limit 1 ").select("id");
        List<DgnsGtgktype> aoidSinglewellConfigs = baseMapper.selectList(aoidSinglewellConfigQueryWrapper);
        if (aoidSinglewellConfigs.size() == 0)
            return 1;
        else
            return aoidSinglewellConfigs.get(0).getId() + 1;
    }

    @Override
    public List<String> getData(String moduleKey, LocalDateTime proTime) {
        return this.list(new QueryWrapper<DgnsGtgktype>()
                        .eq("module_key", moduleKey)
                        .eq("pro_time", proTime))
                .stream()
                .map(DgnsGtgktype::toFileRowString)
                .collect(Collectors.toList());
    }
}
