package com.gct.edge.actuator.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gct.common.core.result.Result;
import com.gct.common.util.DateTimeUtil;
import com.gct.common.util.SGTUtil;
import com.gct.common.util.StringUtil;
import com.gct.edge.actuator.entity.DiagnoseResultRealTime;
import com.gct.edge.actuator.mapper.DiagnoseResultRealTimeMapper;
import com.gct.edge.actuator.service.DiagnoseService;
import com.gct.edge.actuator.service.ServicePlusImpl;
import com.gct.edge.actuator.util.DataTranslator;
import com.gct.edge.base.entity.TableNameEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.InputStream;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * dec: DiagnoseServiceImpl
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/10/24 下午2:08
 */
@Service
public class DiagnoseServiceImpl extends ServicePlusImpl<DiagnoseResultRealTimeMapper, DiagnoseResultRealTime> implements DiagnoseService {

    @Autowired
    DiagnoseResultRealTimeMapper mapper;

    @Override
    public Result getByDate(String moduleKey, String wellId, String date, Integer curPage, Integer pageSize) {
        if (StringUtil.isBlankOrEmpty(moduleKey)) return Result.error("module key must be not null");
        LocalDate curDay = DateTimeUtil.formatDate(date);
        LambdaQueryWrapper<DiagnoseResultRealTime> wrapper = new LambdaQueryWrapper<DiagnoseResultRealTime>()
                .eq(DiagnoseResultRealTime::getModuleKey, moduleKey)
                .between(DiagnoseResultRealTime::getProTime, curDay.atTime(0, 0, 0), curDay.atTime(23, 59, 59))
                .orderByDesc(DiagnoseResultRealTime::getProTime);
        if (!StringUtil.isBlankOrEmpty(wellId)) wrapper.eq(DiagnoseResultRealTime::getWellId, wellId);
//        System.out.println("baseMapper.selectMaps(wrapper) = " + baseMapper.selectMaps(wrapper));
//        System.out.println("selectMapToObject(wrapper) = " + selectMapToObject(wrapper));
//        System.out.println("baseMapper.selectList(wrapper) = " + baseMapper.selectList(wrapper));

        Page<DiagnoseResultRealTime> page = selectPageToObject(new Page(curPage, pageSize), wrapper);// mapper.getListByDateRange(moduleKey, wellId, curDay, curDay.plusDays(1));


        List<JSONObject> records = page.getRecords().stream().map(x -> {
            List<List<Double>> gt = SGTUtil.encodeToUseList(x.getWorkDiagram());
            JSONObject obj = (JSONObject) JSON.toJSON(x);
            obj.put("workDiagram", gt);
            if (!gt.isEmpty()) {
                obj.put("minS", gt.stream().min(Comparator.comparingDouble(p -> p.get(0))).get().get(0));
                obj.put("maxS", gt.stream().max(Comparator.comparingDouble(p -> p.get(0))).get().get(0));
                obj.put("minL", gt.stream().min(Comparator.comparingDouble(p -> p.get(1))).get().get(1));
                obj.put("maxL", gt.stream().max(Comparator.comparingDouble(p -> p.get(1))).get().get(1));
            }
            obj.put("proTime", DateTimeUtil.formatString(x.getProTime()));
            return obj;
        }).collect(Collectors.toList());
        JSONObject res = (JSONObject) JSON.toJSON(page);
        res.put("records", records);
        return Result.ok(res);
    }

    @Override
    public Result loadDataFromFile(InputStream file, String moduleKey, TableNameEnum tableKey) {

        boolean checkParams = true;
        if (StringUtil.isBlankOrEmpty(moduleKey)) checkParams = false;
        if (Objects.isNull(tableKey)) checkParams = false;
        if (!checkParams) return Result.error("input params is not supported");

        List<DiagnoseResultRealTime> data = DataTranslator.transFileToFileParseObject(file, DiagnoseResultRealTime.class);

        data.forEach(x -> {
            x.setModuleKey(moduleKey);
        });

        int sus = saveOnConflictDoNothingBatch(data);
        return sus > 0 ? Result.ok("导入了" + (sus) + "条") : Result.error("导入了0条,检查格式、重复？");
    }

    @Override
    public Result deleteResultByTimeRange(String moduleKey, String beginTime, String endTime) {
        int res = baseMapper.delete(new QueryWrapper<DiagnoseResultRealTime>()
                .eq("module_key", moduleKey)
                .between("pro_time", DateTimeUtil.formatDateTime(beginTime), DateTimeUtil.formatDateTime(endTime)));
        return res > 0 ? Result.ok("删除成功,共" + res + "条") : Result.error("删除失败");
    }

    @Override
    public JSONArray getGtOverlayData(String moduleKey, String wellId, String beginDate, String endDate, Integer splitCount) {
        List<DiagnoseResultRealTime> resultRealTimes = baseMapper.getListByDateRange(moduleKey, wellId, DateTimeUtil.formatDateTime(beginDate), DateTimeUtil.formatDateTime(endDate));
        JSONArray ja = new JSONArray();
        List<DiagnoseResultRealTime> list = new ArrayList<>();
        for (int i = 0; i < resultRealTimes.size(); i++) {
            if (i != 0 && i % splitCount == 0) {
                ja.add(getOverlayObj(list));
                list.clear();
            }
            list.add(resultRealTimes.get(i));
        }
        if (!list.isEmpty()) {
            ja.add(getOverlayObj(list));
        }
        return ja;
    }

    private JSONObject getOverlayObj(List<DiagnoseResultRealTime> list) {
        if (list == null || list.isEmpty()) return null;
        JSONObject obj = new JSONObject();
        List<List<List<Double>>> gtList = list.stream().map(x -> SGTUtil.encodeToUseList(x.getWorkDiagram())).collect(Collectors.toList());
        obj.put("list", gtList);
        obj.put("gtCount", gtList.size());

        obj.put("beginTime", list.get(list.size() - 1).getProTime().format(DateTimeFormatter.ofPattern("MM-dd HH:mm:ss")));
        obj.put("endTime", list.get(0).getProTime().format(DateTimeFormatter.ofPattern("MM-dd HH:mm:ss")));

        List<Double> loadDiffList = gtList
                .stream()
                .map(gt -> gt.stream().max(Comparator.comparingDouble(p -> p.get(1))).get().get(1)
                        - gt.stream().min(Comparator.comparingDouble(p -> p.get(1))).get().get(1)).collect(Collectors.toList());
        obj.put("maxLoadDiff", loadDiffList.stream().max(Double::compareTo).get());
        obj.put("minLoadDiff", loadDiffList.stream().min(Double::compareTo).get());
        return obj;
    }

    @Override
    public Page<DiagnoseResultRealTime> selectPageByCondition(String moduleKey, String wellId, LocalDateTime start, LocalDateTime end, Integer curPage, Integer pageSize) {
        List<Map> maps = mapper.selectShardCount(moduleKey, wellId, start, end);
        Page<DiagnoseResultRealTime> page = new Page<>();
        if (curPage == 0) curPage = 1;
        page.setCurrent(curPage).setTotal(0).setSize(pageSize);
        Set<String> selectKeys = new TreeSet<>();
        int pl = -1;
        int pr = maps.size();
        long destL = (page.getCurrent() - 1) * pageSize + 1;
        long destR = page.getCurrent() * pageSize;

        HashMap<String, Map> shardKeyMap = new HashMap<>();
        for (int i = 0; i < maps.size(); i++) {
            Map map = maps.get(i);
            shardKeyMap.put(map.get("shard_key").toString(), map);
            long shard_count = (long) map.get("shard_total");
            if (page.getTotal() + 1 <= destL && destL <= page.getTotal() + shard_count) {
                pl = i;
//                int offL = ;
                map.put("offset", destL - page.getTotal() - 1);
                map.put("limit", shard_count - (destL - page.getTotal() - 1));
                selectKeys.add(map.get("shard_key").toString());
            }
            if (page.getTotal() + 1 <= destR && destR <= page.getTotal() + shard_count) {
                pr = i;
                if (pr != pl) {
                    map.put("offset", 0);
                    map.put("limit", destR - page.getTotal());
                } else {
                    map.put("limit", page.getSize());
                }
                selectKeys.add(map.get("shard_key").toString());

            }
            page.setTotal(page.getTotal() + shard_count);
        }
        if (pl == -1) return page;

        for (int i = pl + 1; i < pr; i++) {
            Map map = maps.get(i);
            map.put("offset", 0);
            long shard_count = (long) map.get("shard_total");
            map.put("limit", shard_count);
            selectKeys.add(map.get("shard_key").toString());

        }
        page.setPages((int) (Math.ceil(page.getTotal() * 1.0 / page.getSize())));

        List<DiagnoseResultRealTime> records = new ArrayList<>();
        for (String selectKey : selectKeys) {
            LocalDateTime _start = LocalDateTime.parse(selectKey + "01 000000", DateTimeFormatter.ofPattern("yyyyMMdd HHmmss"));
            LocalDateTime _end = _start.plusMonths(1).minusSeconds(1);
            if (_start.isBefore(start)) _start = start;
            if (_end.isAfter(end)) _end = end;
            LambdaQueryWrapper<DiagnoseResultRealTime> wrapper = new LambdaQueryWrapper<DiagnoseResultRealTime>()
                    .eq(DiagnoseResultRealTime::getModuleKey, moduleKey)
                    .eq(DiagnoseResultRealTime::getWellId, wellId)
                    .orderByDesc(DiagnoseResultRealTime::getProTime);
            wrapper.between(DiagnoseResultRealTime::getProTime, _start, _end);
            System.out.println("======= " + JSONObject.toJSONString(shardKeyMap.get(selectKey)));
            wrapper.last(" offset " + shardKeyMap.get(selectKey).get("offset") + " limit " + shardKeyMap.get(selectKey).get("limit"));
            records.addAll(selectMapToObject(wrapper));
        }
        page.setRecords(records);
        return page;
    }

    @Override
    public JSONObject getGtOverlayDataLazy(String moduleKey, String wellId, String beginDate, String endDate, Integer splitCount, Integer curIndex) {
        Page<DiagnoseResultRealTime> data = selectPageToObject(
                new Page<DiagnoseResultRealTime>(curIndex, splitCount).
                        setSearchCount(false),
                new LambdaQueryWrapper<DiagnoseResultRealTime>().eq(DiagnoseResultRealTime::getModuleKey, moduleKey)
                        .eq(DiagnoseResultRealTime::getWellId, wellId)
                        .between(DiagnoseResultRealTime::getProTime,
                                DateTimeUtil.formatDateTime(beginDate), DateTimeUtil.formatDateTime(endDate)));
        int modNum = (int)Math.ceil(splitCount/200.0);
        if (modNum>=2 && data.getRecords().size()>=200){
            List<DiagnoseResultRealTime> filterList = new ArrayList<>();
            for (int i = 0; i < data.getRecords().size(); i++) {
                if (i%modNum==0)filterList.add(data.getRecords().get(i));
            }
            data.setRecords(filterList);
        }
        return getOverlayObj(data.getRecords());
    }
}
