package com.gct.edge.actuator.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gct.common.core.result.Result;
import com.gct.common.util.DateTimeUtil;
import com.gct.edge.actuator.entity.GtcvrRuntimeConfig;
import com.gct.edge.actuator.mapper.GtcvrRuntimeConfigMapper;
import com.gct.edge.actuator.service.GtcvrRuntimeConfigService;
import com.gct.edge.actuator.service.ServicePlusImpl;
import com.gct.edge.actuator.service.SyncRemoteConfig;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author xusirui 2021/10/26
 */
@Service
public class GtcvrRuntimeConfigServiceImpl extends ServicePlusImpl<GtcvrRuntimeConfigMapper, GtcvrRuntimeConfig> implements GtcvrRuntimeConfigService, SyncRemoteConfig {

    @Autowired
    GtcvrRuntimeConfigMapper gtcvrRuntimeConfigMapper;


    /**
     * 保存全局配置表
     *
     * @param
     * @return
     */
    public boolean SaveGtcvrRuntimeConfig(JSONObject jsonObject) {
        String status = jsonObject.getString("status");
        jsonObject.remove("flag");
        jsonObject.remove("status");
        GtcvrRuntimeConfig gtcvrRuntimeConfig = JSON.toJavaObject(jsonObject, GtcvrRuntimeConfig.class);
        UpdateWrapper<GtcvrRuntimeConfig> gtcvrRuntimeConfigUpdateWrapper = new UpdateWrapper<>();
        gtcvrRuntimeConfigUpdateWrapper.eq("module_key", gtcvrRuntimeConfig.getModuleKey())
                .eq("pro_time", gtcvrRuntimeConfig.getProTime());
        if ("update".equals(status)) {
            return gtcvrRuntimeConfigMapper.update(gtcvrRuntimeConfig, gtcvrRuntimeConfigUpdateWrapper) == 1;
        }
        return gtcvrRuntimeConfigMapper.insert(gtcvrRuntimeConfig) == 1;
    }


    /**
     * 根据参数条件查询全局配置数据
     *
     * @param pageNum
     * @param pageSize
     * @return
     */

    public Result findAllGtCvrRuntimeConfig(Integer pageNum, Integer pageSize, String modelKey, String protime) {
        LocalDateTime dateTime = DateTimeUtil.formatDateTime(protime);
        Page<Map<String, Object>> paramPage = new Page<>(pageNum, pageSize);
        paramPage.offset();
        //查询条件
        QueryWrapper<GtcvrRuntimeConfig> queryWrapper = new QueryWrapper<>();
        QueryWrapper<GtcvrRuntimeConfig> wrapper = queryWrapper
                .eq("module_key", modelKey)
                .lt("pro_time", dateTime);
        Page<Map<String, Object>> page = baseMapper.selectMapsPage(paramPage, wrapper);
        return Result.ok(page);

    }

    @Override
    public List<GtcvrRuntimeConfig> getMostNewGtCvrRuntimeConfig(String modelKey, String releaseVersion) {
        if ("".equals(modelKey) || "".equals(releaseVersion)) return null;
        QueryWrapper<GtcvrRuntimeConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("module_key", modelKey)
                .eq("pro_time", DateTimeUtil.formatDateTime(releaseVersion));
        return baseMapper.selectList(queryWrapper);
    }

    @Override
    public Map<String, Object> copyGlobalConfig(JSONObject jsonObject) {
        LocalDateTime now = LocalDateTime.now();
        String status = jsonObject.getString("status");
        String moduleKey = jsonObject.getString("moduleKey");
        String targetModuleKey = jsonObject.getString("targetModuleKey");
        String release_version = jsonObject.getString("relsease_version");
        QueryWrapper<GtcvrRuntimeConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("module_key", moduleKey)
                .le("pro_time", now.plusDays(1))
                .last("order by pro_time desc limit 1");
        List<GtcvrRuntimeConfig> gtcvrRuntimeConfigs = baseMapper.selectList(queryWrapper);
        GtcvrRuntimeConfig gtcvrRuntimeConfig = gtcvrRuntimeConfigs.get(0);
        gtcvrRuntimeConfig.setModuleKey(targetModuleKey);
        gtcvrRuntimeConfig.setProTime(DateTimeUtil.formatDateTime(release_version.replace("T", " ")));
        if ("update".equals(status)) {
            UpdateWrapper<GtcvrRuntimeConfig> gtcvrRuntimeConfigUpdateWrapper = new UpdateWrapper<>();
            gtcvrRuntimeConfigUpdateWrapper.eq("module_key", moduleKey).eq("pro_time", release_version.replace("T", " "));
            gtcvrRuntimeConfigMapper.update(gtcvrRuntimeConfig, gtcvrRuntimeConfigUpdateWrapper);
        } else {
            int insert = baseMapper.insert(gtcvrRuntimeConfig);

        }
        return null;
    }

    @Override
    public boolean deleteGlobalConfig(String moduleKey, String proTime) {

        QueryWrapper<GtcvrRuntimeConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("module_key", moduleKey)
                .eq("pro_time", DateTimeUtil.formatDateTime(proTime.replace("T", " ")));
        int delete = baseMapper.delete(queryWrapper);
        return delete == 1;
    }

    @Override
    public List<String> getData(String moduleKey, LocalDateTime proTime) {
        return this.list(new QueryWrapper<GtcvrRuntimeConfig>()
                        .eq("module_key", moduleKey)
                        .eq("pro_time", proTime))
                .stream()
                .map(GtcvrRuntimeConfig::toFileRowString)
                .collect(Collectors.toList());
    }
}
