package com.gct.edge.actuator.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.gct.common.core.result.Result;
import com.gct.edge.actuator.config.SyncModuleDataProperties;
import com.gct.edge.actuator.remoteDataStruct.SqlGenerate;
import com.gct.edge.actuator.service.SyncModuleConfigService;
import com.gct.edge.actuator.service.SyncRemoteConfig;
import com.gct.edge.actuator.util.SqlGeneratorUtil;
import com.gct.edge.actuator.util.SshUtil;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.stereotype.Service;

import java.io.*;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Locale;

/**
 * @author xusirui 2021/12/3
 */
@Service
public class SyncModuleConfigServiceImpl implements SyncModuleConfigService, ApplicationContextAware {

    ApplicationContext applicationContext;

    @Autowired
    SyncModuleDataProperties properties;

    @Override
    public Result syncModuleConfig(String moduleKey, LocalDateTime proTime, String ipAddress,
                                   Integer port, String authUser, String authPassword,
                                   String remotePath, String tableName, String actualFileName) {
        String localFile = properties.getPath() + "/" + moduleKey + "/" + actualFileName;
        File file = new File(localFile);
        if (!file.getParentFile().exists()) file.getParentFile().mkdirs();

        try (BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file)))) {
            String[] entityNames = tableName.split("\\_");
            StringBuilder entityName = new StringBuilder();
            for (int i = 0; i < entityNames.length; i++) {
                if (i != 0)
                    entityNames[i] = entityNames[i].substring(0, 1).toUpperCase(Locale.ROOT) + entityNames[i].substring(1);
                entityName.append(entityNames[i]);
            }
            SyncRemoteConfig syncRemoteConfig = applicationContext.getBean(entityName + "ServiceImpl", SyncRemoteConfig.class);
            List<String> data = syncRemoteConfig.getData(moduleKey, proTime);
            for (int i = 0; i < data.size(); i++) {
                bw.write(data.get(i));
                if (i!=data.size()-1)bw.write('\n');
            }
            bw.flush();
            SshUtil.sendLocalFile(authUser, authPassword, ipAddress, port, localFile, remotePath);
        } catch (Exception e) {
            e.printStackTrace();
            return Result.error(e.getMessage());
        }
        return Result.ok("成功");
    }

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        this.applicationContext = applicationContext;
    }
}
