package com.gct.edge.actuator.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gct.common.core.result.Result;
import com.gct.common.util.DateTimeUtil;

import com.gct.common.util.StringUtil;
import com.gct.edge.actuator.entity.AoidProdState;
import com.gct.edge.actuator.entity.YieldDaily;
import com.gct.edge.actuator.entity.YieldRealTime;
import com.gct.edge.actuator.mapper.YieldDailyMapper;
import com.gct.edge.actuator.service.AoidProdStateService;
import com.gct.edge.actuator.service.ServicePlusImpl;
import com.gct.edge.actuator.service.YieldDailyService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.Duration;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.*;
import java.util.stream.Collectors;

import com.gct.edge.actuator.service.YieldService;

/**
 * @author xusirui 2021/10/26
 */
@Service
public class YieldDailyServiceImpl extends ServicePlusImpl<YieldDailyMapper, YieldDaily> implements YieldDailyService {

    @Autowired
    YieldService yieldService;

    @Autowired
    YieldDailyMapper yieldDailyMapper;

    @Autowired
    AoidProdStateService aoidProdStateService;

    @Override
    public List<YieldDaily> getAoidDailyYieldInfoByWellId(String moduleKey, String wellName, String startDate) {

        LocalDateTime start = DateTimeUtil.formatDateTime(startDate + " 00:00:00");
        LocalDateTime end = DateTimeUtil.formatDateTime(startDate + " 23:59:59");

        LambdaQueryWrapper<YieldDaily> queryWrapper = new LambdaQueryWrapper<YieldDaily>()
                .eq(YieldDaily::getModuleKey, moduleKey)
                .eq(YieldDaily::getWellCommonName, wellName)
                .ge(YieldDaily::getProTime, start).
                le(YieldDaily::getProTime, end).orderByAsc(YieldDaily::getProTime);
        return yieldDailyMapper.selectList(queryWrapper);
    }

    @Override
    public Result timingSummary(String moduleKey, String startDate, String endDate) {
        QueryWrapper<YieldDaily> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("moduleKey", moduleKey);
        queryWrapper.gt("pro_time", startDate);
        queryWrapper.lt("pro_time", startDate);
        return Result.ok(baseMapper.selectList(queryWrapper));
    }

    @Override
    public Result calculate(String moduleKey, String wellIdList, String startTime) {
        try {

            if (calculate_yield_cal(moduleKey, wellIdList, startTime))
                return Result.ok(moduleKey + "，" + startTime + ",日产量计算完成");
        } catch (Exception e) {
            return Result.error(e.getMessage());
        }
        return Result.error("Unknown error.");
    }

    private double calProdTime(List<AoidProdState> states) {

        if (Objects.isNull(states) || states.size() < 2) return 0;
        double openTotalTime = 0d;
        AoidProdState last = states.get(0);
        for (int i = 1; i < states.size(); i++) {
            AoidProdState state = states.get(i);
            if (state.getProdState() == 1) {//当前为开状态，则减去上一个状态时间，累计为开井时间
                double t = Duration.between(last.getProTime(), state.getProTime()).toMillis() / 1000.0;//s
                if (t > 0) openTotalTime += t;
            }
            last = state;
        }
        openTotalTime = openTotalTime / 60 / 60;
        openTotalTime = Math.ceil(openTotalTime);//去整小时
        if (openTotalTime > 24) openTotalTime = 24;

        return openTotalTime;//h
    }

    private boolean calculate_yield_cal(String moduleKey, String wellIdList, String startTime) {
        JSONArray wells = JSON.parseArray(wellIdList);
        for (Object well : wells) {
            String wellId = ((JSONObject) well).getString("wellId");
            String wellName = ((JSONObject) well).getString("wellName");
            List<YieldRealTime> lis = yieldService.getYieldRealTimeInfoByWellId(moduleKey, wellId, startTime);
            List<AoidProdState> states = aoidProdStateService.getAoidProdStateList(moduleKey, wellId, startTime);
            double prodTotalTime = calProdTime(states);
            calculate_yield(lis, startTime, wellName, moduleKey, wellId, prodTotalTime);


        }
        return true;
    }

    private void setRemark(YieldDaily re, String newRemark) {
        re.setBz((StringUtil.isBlankOrEmpty(re.getBz()) ? "" : re.getBz() + ";") + newRemark);
    }

    private void calculate_yield(List<YieldRealTime> lis, String startTime, String wellName, String moduleKey, String wellId, Double prodTime) {
        //必要的初始化
        YieldDaily re = new YieldDaily() {
            {
                setBz("");
                setModuleKey(moduleKey);
                setProTime(DateTimeUtil.formatDateTime(startTime + " 00:00:00"));
                setProdDate(DateTimeUtil.formatDateTime(startTime + " 00:00:00"));

                //region prod time

//                if (lis.size() > 0) {
//                    LocalDateTime prodStart = lis.get(0).getProTime();
//                    LocalDateTime prodEnd = lis.get(lis.size() - 1).getProTime();
//                    double time = (prodEnd.toEpochSecond(ZoneOffset.of("+8")) - prodStart.toEpochSecond(ZoneOffset.of("+8"))) / 3_600.0;
//                    setProdTime((int) Math.ceil(time));
//                } else setProdTime(0);
//
//
//                if (getProdTime() >= 20) {
//                    setProdTime(24);
//                }
                setProdTime(prodTime.intValue());//使用开井状态值
                //endregion

                setWellCommonName(wellName);
                setWellId(wellId);
            }
        };
        //汇总现在是实时汇总,当天数据当天汇总

        int calNormalCount = 0;
        int strokeDataCount = 0;
        int frequencyDataCount = 0;
        Set<String> distinctZdjg = new HashSet<>();

        for (YieldRealTime item : lis) {
            if (calNormalCount == 0) {
                re.setLiqProdDaily(item.getLiqProdDaily());
                re.setOilProdDaily(item.getOilProdDaily());
                re.setGasProdDaily(item.getGasProdDaily());
                re.setWaterProdDaily(item.getWaterProdDaily());
                re.setWaterCut(item.getMoistureContent());
                re.setBx(item.getPumpEfficiency());
                re.setLlpl(item.getTheoryDisplacement());
            } else {
                re.setLiqProdDaily((re.getLiqProdDaily() * calNormalCount / (calNormalCount + 1)) + (item.getLiqProdDaily() / (calNormalCount + 1)));
                re.setOilProdDaily((re.getOilProdDaily() * calNormalCount / (calNormalCount + 1)) + (item.getOilProdDaily() / (calNormalCount + 1)));
                re.setGasProdDaily((re.getGasProdDaily() * calNormalCount / (calNormalCount + 1)) + (item.getGasProdDaily() / (calNormalCount + 1)));
                re.setWaterProdDaily((re.getWaterProdDaily() * calNormalCount / (calNormalCount + 1)) + (item.getWaterProdDaily() / (calNormalCount + 1)));
                //re.setWaterCut(re.getWaterCut() * calNormalCount / (calNormalCount + 1) + item.getMoistureContent() / (calNormalCount + 1));
                re.setWaterCut(item.getMoistureContent());//不用平均
                re.setBx(re.getBx() * calNormalCount / (calNormalCount + 1) + item.getPumpEfficiency() / (calNormalCount + 1));
                re.setLlpl(re.getLlpl() * calNormalCount / (calNormalCount + 1) + item.getTheoryDisplacement() / (calNormalCount + 1));
            }
            calNormalCount++;


            //region 冲程冲次
            if (Objects.nonNull(item.getStroke()) && item.getStroke() > 0 && item.getStroke() <= 100) {
                if (strokeDataCount == 0) re.setStroke(item.getStroke());
                else
                    re.setStroke(re.getStroke() * strokeDataCount / (strokeDataCount + 1) + item.getStroke() / (strokeDataCount + 1));
                strokeDataCount++;
            }
            if (Objects.nonNull(item.getFrequence()) && item.getFrequence() > 0 && item.getFrequence() <= 100) {
                if (frequencyDataCount == 0) re.setFrequence(item.getFrequence());
                else
                    re.setFrequence(re.getFrequence() * frequencyDataCount / (frequencyDataCount + 1) + item.getFrequence() / (frequencyDataCount + 1));
                frequencyDataCount++;
            }


            if (Objects.nonNull(item.getResult2()))
                distinctZdjg.add(item.getResult2());
            //endregion

            if (Objects.nonNull(item.getPumpDiameter())) {
                re.setBj(item.getPumpDiameter());
                re.setYz(item.getPumpDiameter());
            }
            if (Objects.nonNull(item.getPumpDepth())) re.setBs(item.getPumpDepth());
            if (Objects.nonNull(item.getSubmergence())) re.setCmd(item.getSubmergence());
            if (Objects.nonNull(re.getBs()) && Objects.nonNull(re.getCmd()))
                re.setDym(Math.ceil(re.getBs() - re.getCmd()));
            if (Objects.nonNull(item.getGasLiqRatio())) re.setQyb(item.getGasLiqRatio());
            //endregion
        }
        if (calNormalCount == 0) setRemark(re, "功图数量不足");

        re.setZdjg(StringUtils.join(distinctZdjg.toArray(), ';'));

        if (re.getLiqProdDaily() != null && re.getWaterCut() != null) {
            if (Objects.isNull(re.getProdTime()) || re.getProdTime() == 0) {
                re.setProdTime(0);
                setRemark(re, "生产时间0");
                re.setLiqProdDaily(0d);
                re.setWaterProdDaily(0d);
                re.setOilProdDaily(0d);
            } else {
                double rate = re.getProdTime() / 24d;
                re.setLiqProdDaily(re.getLiqProdDaily() * rate);
                re.setOilProdDaily(re.getOilProdDaily() * rate);
                re.setWaterProdDaily(re.getWaterProdDaily() * rate);
                re.setGasProdDaily(re.getGasProdDaily() * rate);

//                re.setWaterProdDaily(Math.round((re.getLiqProdDaily() * re.getWaterCut() / 10d)) / 10d);
//                re.setOilProdDaily(Math.round((re.getLiqProdDaily() - re.getWaterProdDaily()) * 10) / 10d);
//                if (re.getQyb() != null)
//                    re.setGasProdDaily(Math.round((re.getOilProdDaily() * re.getQyb()) * 10) / 10d);
            }
//            re.setWaterCut(Math.round(re.getWaterCut() * 10) / 10d);
        }
        //region 格式化
        if (Objects.nonNull(re.getFrequence())) re.setFrequence(Math.round(re.getFrequence() * 10) / 10d);
        if (Objects.nonNull(re.getStroke())) re.setStroke(Math.round(re.getStroke() * 10) / 10d);
        if (Objects.nonNull(re.getWaterProdDaily()))
            re.setWaterProdDaily(Math.round(re.getWaterProdDaily() * 10) / 10d);
        if (Objects.nonNull(re.getOilProdDaily())) re.setOilProdDaily(Math.round(re.getOilProdDaily() * 10) / 10d);
        if (Objects.nonNull(re.getGasProdDaily())) re.setGasProdDaily(Math.round(re.getGasProdDaily() * 10) / 10d);
        if (Objects.nonNull(re.getLiqProdDaily())) re.setLiqProdDaily(Math.round(re.getLiqProdDaily() * 10) / 10d);
        if (Objects.nonNull(re.getBx())) re.setBx(Math.round(re.getBx() * 10) / 10d);
        if (Objects.nonNull(re.getLlpl())) re.setLlpl(Math.round(re.getLlpl() * 10) / 10d);
        if (Objects.nonNull(re.getWaterCut())) re.setWaterCut(Math.round(re.getWaterCut() * 10) / 10d);
        //endregion
        YieldDaily one = baseMapper.selectOne(new LambdaQueryWrapper<YieldDaily>()
                .eq(YieldDaily::getModuleKey, re.getModuleKey())
                .eq(YieldDaily::getWellId, re.getWellId())
                .eq(YieldDaily::getProTime, re.getProTime()));
        if (Objects.isNull(one)) save(re);
        else update(re, new LambdaUpdateWrapper<YieldDaily>()
                .eq(YieldDaily::getModuleKey, re.getModuleKey())
                .eq(YieldDaily::getWellId, re.getWellId())
                .eq(YieldDaily::getProTime, re.getProTime()));


    }

    @Override
    public Page getWellProdData(String moduleKey, String wellId, String beginDate, String endDate, Integer curPage, Integer pageSize) {
        QueryWrapper<YieldDaily> wrapper = new QueryWrapper<YieldDaily>()
                .eq("module_key", moduleKey)
                .between("pro_time", DateTimeUtil.formatDate(beginDate).atTime(0, 0, 0),
                        DateTimeUtil.formatDate(endDate).atTime(0, 0, 0))
                .last("order by pro_time desc");
        if (!StringUtil.isBlankOrEmpty(wellId)) wrapper = wrapper.eq("well_id", wellId);
        QueryWrapper<YieldDaily> wrapper2 = new QueryWrapper<YieldDaily>()
                .select("count(1) shard_total")
                .eq("module_key", moduleKey)
                .between("pro_time", DateTimeUtil.formatDate(beginDate).atTime(0, 0, 0)
                        , DateTimeUtil.formatDate(endDate).atTime(0, 0, 0));
        if (!StringUtil.isBlankOrEmpty(wellId)) wrapper2 = wrapper2.eq("well_id", wellId);
        List<Map<String, Object>> count = baseMapper.selectMaps(wrapper2);

        Page pageSetting = new Page<YieldDaily>(curPage, pageSize).setSearchCount(false);
        Page<YieldDaily> page = selectPageToObject(pageSetting, wrapper);
        long shard_total = Long.parseLong(count.get(0).get("shard_total").toString());
        page.setTotal(shard_total);
        page.setPages((long) Math.ceil(shard_total / page.getSize()));

        return page;
    }

    @Override
    public List<YieldDaily> getWellProdData(String moduleKey, String wellId, String beginDate, String endDate) {
        QueryWrapper<YieldDaily> wrapper = new QueryWrapper<YieldDaily>()
                .eq("module_key", moduleKey)
                .between("pro_time", DateTimeUtil.formatDate(beginDate).atTime(0, 0, 0),
                        DateTimeUtil.formatDate(endDate).atTime(0, 0, 0))
                .last("order by pro_time desc");
        if (!StringUtil.isBlankOrEmpty(wellId)) wrapper = wrapper.eq("well_id", wellId);
        return this.list(wrapper);
    }

    @Override
    public JSONArray getWellProdGraph(String moduleKey, String wellId, String beginDate, String endDate) {
        QueryWrapper<YieldDaily> wrapper = new QueryWrapper<YieldDaily>()
                .eq("module_key", moduleKey)
                .between("pro_time", DateTimeUtil.formatDate(beginDate).atTime(0, 0, 0),
                        DateTimeUtil.formatDate(endDate).atTime(0, 0, 0))
                .last("order by pro_time asc");
        List<YieldDaily> list = baseMapper.selectList(wrapper);
        Map<String, List<YieldDaily>> map = new HashMap<>();
        //分组
        list.forEach(x -> map.putIfAbsent(x.getWellId(), new ArrayList<>()));
        list.forEach(x -> map.get(x.getWellId()).add(x));
        JSONArray res = new JSONArray();
        map.forEach((k, v) -> {
            JSONObject obj = new JSONObject();
            obj.put("wellName", v.get(0).getWellCommonName());
            obj.put("liqProdDailyArr", v.stream().map(x -> new Object[]{x.getProTime().toEpochSecond(ZoneOffset.of("+8")) * 1000, x.getLiqProdDaily()}).collect(Collectors.toList()));
            obj.put("oilProdDailyArr", v.stream().map(x -> new Object[]{x.getProTime().toEpochSecond(ZoneOffset.of("+8")) * 1000, x.getOilProdDaily()}).collect(Collectors.toList()));
            res.add(obj);
        });
        return res;
    }
}
