package com.gct.edge.actuator.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gct.common.core.result.Result;
import com.gct.common.util.DateTimeUtil;
import com.gct.common.util.StringUtil;
import com.gct.edge.actuator.algorithm.CalIntegral;
import com.gct.edge.actuator.entity.YieldRealTime;
import com.gct.edge.actuator.mapper.YieldRealTimeMapper;
import com.gct.edge.actuator.service.ServicePlusImpl;
import com.gct.edge.actuator.service.YieldService;
import com.gct.edge.actuator.util.DataTranslator;
import com.gct.edge.base.entity.TableNameEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.io.InputStream;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author xusirui 2021/10/26
 */
@Service
public class YieldServiceImpl extends ServicePlusImpl<YieldRealTimeMapper, YieldRealTime> implements YieldService {

    @Autowired
    public YieldRealTimeMapper yieldRealTimeMapper;

    private final DateTimeFormatter formatterDateTime = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
    private final String time0 = "00:00:00";
    private final String time1 = "23:59:59";


    @Override
    public Result loadDataFromFile(InputStream file, String moduleKey, TableNameEnum tableKey) throws IOException {
        boolean checkParams = !StringUtil.isBlankOrEmpty(moduleKey);
        if (Objects.isNull(tableKey)) checkParams = false;
        if (!checkParams) return Result.error("input params is not supported");

        List<YieldRealTime> data = DataTranslator.transFileToFileParseObject(file, YieldRealTime.class);
        data.forEach(x -> {
            x.setModuleKey(moduleKey);
        });
        int sus = saveOnConflictDoNothingBatch(data);
//        int failed = 0;
//        for (int i = 0; i < data.size(); i++) {
//            try{
//                this.save(data.get(i));
//            }catch (Exception e){
//                failed++;
//            }
//        }
        return sus > 0 ? Result.ok("导入了" + (sus) + "条") : Result.error("导入了0条,检查格式、重复？");
    }

    @Override
    public JSONObject getProdCurve(String moduleKey, String date, String wellId) {
        List<YieldRealTime> list = baseMapper.selectList(new QueryWrapper<YieldRealTime>()
                .eq("module_key", moduleKey)
                .between("pro_time", DateTimeUtil.formatDate(date).atTime(0, 0, 0),
                        DateTimeUtil.formatDate(date).atTime(23, 59, 59))
                .eq("well_id", wellId)
                .orderByAsc("pro_time"));
        JSONObject obj = new JSONObject();
        obj.put("liqProdDailyArr", list.stream()
                .map(x -> new Object[]{x.getProTime().toEpochSecond(ZoneOffset.of("+8")) * 1000, x.getLiqProdDaily()})
                .collect(Collectors.toList()));
        obj.put("oilProdDailyArr", list.stream()
                .map(x -> new Object[]{x.getProTime().toEpochSecond(ZoneOffset.of("+8")) * 1000, x.getOilProdDaily()})
                .collect(Collectors.toList()));
       /* obj.put("proTimeArr", list.stream()
                .map(x -> x.getProTime().toEpochSecond(ZoneOffset.of("+8")))
                .collect(Collectors.toList()));*/
        obj.put("liqIntegral", CalIntegral.cal(
                list.stream().map(x -> new Object[]{x.getProTime().toEpochSecond(ZoneOffset.of("+8")) * 1000, x.getLiqProdDaily()}).collect(Collectors.toList()).toArray(new Object[list.size()][])
        ));
        obj.put("oilIntegral", CalIntegral.cal(
                list.stream().map(x -> new Object[]{x.getProTime().toEpochSecond(ZoneOffset.of("+8")) * 1000, x.getOilProdDaily()}).collect(Collectors.toList()).toArray(new Object[list.size()][])
        ));

        return obj;
    }

    @Override
    public Result getWellIdsByNew(String moduleKey, String startDate) {

        LocalDateTime start = DateTimeUtil.formatDateTime(startDate +" 00:00:00");
        LocalDateTime end = DateTimeUtil.formatDateTime(startDate +" 23:59:59");

        QueryWrapper<YieldRealTime> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("module_key",moduleKey);
        queryWrapper.gt("pro_time",start);
        queryWrapper.lt("pro_time",end);
        return Result.ok( baseMapper.selectList(queryWrapper));

    }


    @Override
    public List<YieldRealTime> getYieldRealTimeInfoByWellId(String moduleKey, String wellId, String startdate) {

        LocalDateTime start = DateTimeUtil.formatDateTime(startdate +" 00:00:00");
        LocalDateTime end = DateTimeUtil.formatDateTime(startdate +" 23:59:59");

        LambdaQueryWrapper<YieldRealTime> queryWrapper = new LambdaQueryWrapper<YieldRealTime>()
                .eq(YieldRealTime::getModuleKey,moduleKey)
                .between(YieldRealTime::getProTime,start,end)
                .eq(YieldRealTime::getWellId,wellId)
                .orderByAsc(YieldRealTime::getProTime);
        List<YieldRealTime> maps = yieldRealTimeMapper.selectList(queryWrapper);

        return maps;
    }

    private Double makeEffectiveMath(Double num,int effect){
        return Math.ceil(num*Math.pow(10,effect))/(int)Math.pow(10,effect);
    }

    @Override
    public JSONArray getWellInfo(String moduleKey, String wellId, String prodTime) {
        YieldRealTime yieldRealTime = baseMapper.selectOne(new QueryWrapper<YieldRealTime>()
                .eq("module_key", moduleKey)
                .eq("well_id", wellId)
                .between("pro_time", DateTimeUtil.formatDate(prodTime).atTime(0,0,0),
                        DateTimeUtil.formatDate(prodTime).atTime(23,59,59)
                )
                .orderByDesc("pro_time")
                .last("limit 1")
        );
        JSONArray ja = new JSONArray();
        if (!Objects.isNull(yieldRealTime)) {
            ja.add(infoObj("冲程：", makeEffectiveMath(yieldRealTime.getStroke(),2)+" m","冲次：", makeEffectiveMath(yieldRealTime.getFrequence(),2)));
            ja.add(infoObj("泵深：", makeEffectiveMath(yieldRealTime.getPumpDepth(),1)+" m","泵径：", makeEffectiveMath(yieldRealTime.getPumpDiameter(),1)+" mm"));
            ja.add(infoObj("动液面：", makeEffectiveMath((yieldRealTime.getPumpDepth() - yieldRealTime.getSubmergence()),1)+" m","含水率：", makeEffectiveMath(yieldRealTime.getMoistureContent(),1) + " %"));
            ja.add(infoObj("沉没度：", makeEffectiveMath(yieldRealTime.getSubmergence(),1)+" m","气液比：", makeEffectiveMath(yieldRealTime.getGasLiqRatio(),1)));
        }
        return ja;
    }

    private JSONObject infoObj(String name, Object val,String name2,Object val2) {
        JSONObject obj = new JSONObject();
        obj.put("name", name);
        obj.put("value", val);
        obj.put("name2", name2);
        obj.put("value2", val2);
        return obj;
    }
}
