package com.gct.edge.actuator.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gct.common.util.DateTimeUtil;
import com.gct.edge.actuator.EdgeModuleActuatorApplication;
import com.gct.edge.actuator.entity.DiagnoseResultRealTime;
import com.gct.edge.actuator.service.DiagnoseService;
import com.gct.edge.actuator.service.YieldDailyService;
import com.gct.edge.actuator.service.YieldService;
import lombok.extern.slf4j.Slf4j;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;

@Slf4j
@SpringBootTest(classes = EdgeModuleActuatorApplication.class)
@RunWith(SpringRunner.class)
@ContextConfiguration
public class DiagnoseServiceImplTest {

    @Autowired
    DiagnoseService service;

    @Autowired
    YieldService  yieldService;

    @Autowired
    YieldDailyService yieldDailyService;

    @Test
    public void test0() {
        LocalDateTime start = DateTimeUtil.formatDateTime("2021-10-01 00:00:00");
        LocalDateTime end = DateTimeUtil.formatDateTime("2021-12-01 00:00:00");
        LambdaQueryWrapper<DiagnoseResultRealTime> wrapper = new LambdaQueryWrapper<DiagnoseResultRealTime>()
                .eq(DiagnoseResultRealTime::getModuleKey, "module1")
                .between(DiagnoseResultRealTime::getProTime, start, end)
                .eq(DiagnoseResultRealTime::getWellId, "1")
                .orderByDesc(DiagnoseResultRealTime::getProTime);

        List<Map<String, Object>> count = service.getBaseMapper().selectMaps(new QueryWrapper<DiagnoseResultRealTime>()
                .select("count(1) shard_total")
                .eq("module_key", "module1")
                .eq("well_id", "1")
                .between("pro_time", start, end)
        );

        Page pageSetting = new Page<DiagnoseResultRealTime>(1, 10).setSearchCount(false);
        Page page = service.getBaseMapper().selectMapsPage(pageSetting, wrapper);
        Long shard_total = Long.valueOf(count.get(0).get("shard_total").toString());
        page.setTotal(shard_total);
        page.setPages((long) Math.ceil(shard_total / page.getSize()));
        System.out.println("page = " + JSONObject.toJSONString(page));
    }

    @Test
    public void test() {
        LocalDateTime start = DateTimeUtil.formatDateTime("2021-10-01 00:00:00");
        LocalDateTime end = DateTimeUtil.formatDateTime("2021-12-01 00:00:00");
        LambdaQueryWrapper<DiagnoseResultRealTime> wrapper = new LambdaQueryWrapper<DiagnoseResultRealTime>()
                .eq(DiagnoseResultRealTime::getModuleKey, "module1")
                .between(DiagnoseResultRealTime::getProTime, start, end)
                .orderByDesc(DiagnoseResultRealTime::getProTime);
        Page<DiagnoseResultRealTime> pageSetting = new Page<DiagnoseResultRealTime>(1, 10).setSearchCount(false);
//        service.selectMapsPage(pageSetting,wrapper);
        //List<Map<String, Object>> maps = service.getBaseMapper().selectMaps(wrapper);
        QueryWrapper<DiagnoseResultRealTime> wrapper1 = new QueryWrapper<DiagnoseResultRealTime>()
                .select("count(1) shard_total,to_char(pro_time,'yyyyMM') shard_key")
                .eq("module_key", "module1")
                .between("pro_time", start, end)
                .groupBy("to_char(pro_time,'yyyyMM')");
        List<Map<String, Object>> diagnoseResultRealTimes = service.getBaseMapper().selectMaps(wrapper1);
        Integer integer = service.getBaseMapper().selectCount(wrapper);
        Page<DiagnoseResultRealTime> page = service.getBaseMapper().selectPage(new Page<DiagnoseResultRealTime>(1, 20), wrapper);


    }

    @Test
    public void test2() {
        LocalDateTime start = DateTimeUtil.formatDateTime("2021-10-01 00:00:00");
        LocalDateTime end = DateTimeUtil.formatDateTime("2021-12-01 00:00:00");
        Page<DiagnoseResultRealTime> module1 = service.selectPageByCondition("module1", "1", start, end, 2, 10);
        System.out.println("module1.getRecords().size() = " + module1.getRecords().size());
        System.out.println("module1 = " + JSONObject.toJSONString(module1));
    }

    @Test
    public void test5() {

        String moduleKey="module4";
        String wellidlist="[{\"wellId\":\"1\",\"wellName\":\"ddd\"},{\"wellId\":\"3\",\"wellName\":\"ewer\"}]";
//        String wellid="1";
        String date ="2021-11-17";
//        List<AoidDailyYield> ss = yieldDailyService.getAoidDailyYieldInfoByWellId(moduleKey,wellid,date);
//        System.out.println(ss.size());


        yieldDailyService.calculate(moduleKey,wellidlist,date);

    }

    public static void main(String[] args) {

        Integer[] m = {1, 87};// [1,7] [8,15] [16,19] [20,22]
        int pageSize = 10;
        for (int curPage = 0; curPage <= 4; curPage++) {
            int total = 0;
            int destL = curPage * pageSize + 1;
            int destR = (curPage + 1) * pageSize;
            int pl = -1, pr = -1, offL = -1, offR = -1;
            for (int i = 0; i < m.length; i++) {
                if (total + 1 <= destL && destL <= total + m[i]) {
                    pl = i + 1;
                    offL = destL - total;
                }
                if (total + 1 <= destR && destR <= total + m[i]) {
                    pr = i + 1;
                    offR = destR - total;
                }
                total += m[i];
            }
            System.out.print("pl = " + pl);
            System.out.print(", offL = " + offL);
            System.out.print(", pr = " + pr);
            System.out.print(", offR = " + offR);
            System.out.println("");
        }

    }
}